<?php

namespace Ems\Services;

use Util\Environment;
use Ems\Repository\IndicadoresSegurancaTrabalhoRepository;
use Ems\Repository\OcorrenciaRepository;
use Ems\Services\EstruturaService;
use Ems\Util\ArrayUtil;
use Util\DateUtil;
use Ems\Util\Utils;

class IndicadoresSegurancaTrabalhoService {

    /** @var Environment */
    protected $environment;

    /** @var IndicadoresSegurancaTrabalhoRepository */
    protected $indicadoresSegurancaTrabalhoRepository;

    /** @var OcorrenciaRepository */
    protected $ocorrenciaRepository;

    /** @var EstruturaService */
    protected $estruturaService;
    
    public function __construct(
        Environment                            $environment,
        IndicadoresSegurancaTrabalhoRepository $indicadoresSegurancaTrabalhoRepository,
        OcorrenciaRepository                   $ocorrenciaRepository,
        EstruturaService                       $estruturaService
    )
    {
        $this->environment                            = $environment;
        $this->indicadoresSegurancaTrabalhoRepository = $indicadoresSegurancaTrabalhoRepository;
        $this->ocorrenciaRepository                   = $ocorrenciaRepository;
        $this->estruturaService                       = $estruturaService;
    }

    private function verificaPerfilAcesso($row)
    {
        $perfilSesmt = null;
        $nrEstruturasString = null;

        $perfilSesmt = $this->ocorrenciaRepository->verificaPerfilAcessoSESMT($row['NRORG'], $row['CDOPERADOR']);
        if(!empty($perfilSesmt)) {
            $row['NRESTRUTLEGAL'] = null;
            return $row;
        } else {
            if ($row['NRESTRUTURAM'] !== "-1") {
                $nrEstruturas = $this->estruturaService->getTodasEstruturasInferiores($row['NRESTRUTURAM'], true);
    
                $nrEstruturasString = ArrayUtil::toStringSQL($nrEstruturas);
                $row['NRESTRUTGEREN'] = $nrEstruturasString;
                return $row;
            }
            $row['NRESTRUTLEGAL'] = null;
            return $row;
        }

        return $row;
    }

    private function adicionaCondicao($row) 
    {

        $row['NRTIPOESTRUTGEREN'] = $this->environment->getTipoEstruturaGerencialOrg();
        $row['NRTIPOESTRUTOMADOR'] =  $this->environment->getTipoEstruturaTomadorOrg();

        return $row;
    }


    public function getTotalAcidentes($row)
    {   
        $totais = array();
        $row['NRESTRUTLEGAL'] = null;
        $row['NRESTRUTGEREN'] = null;
        $row = $this->verificaPerfilAcesso($row);

        $row = $this->adicionaCondicao($row);

        if(!isset($row['DTMESCOMPETENC'])) {
            $row['DTMESCOMPETENC'] = Utils::getParamters()->getDtmescomptencDiaCorrente();

            $totais = $this->indicadoresSegurancaTrabalhoRepository->getTotalAcidentesDashboard(
                $row
            );
            
            return $totais;
        }

        $totais = $this->indicadoresSegurancaTrabalhoRepository->getTotalAcidentes(
            $row
        );

        return $totais;
    }

    public function getTotalAcidentesGerenOcup($row)
    {   
        $totais = array();
        $row['NRESTRUTLEGAL'] = null;
        $row['NRESTRUTGEREN'] = null;
        $row = $this->verificaPerfilAcesso($row);
        $row = $this->adicionaCondicao($row);

        
        $totais = $this->indicadoresSegurancaTrabalhoRepository->getTotalAcidentesGerenOcup(
            $row
        );

        return $totais;
    }

    public function getTotalAcidentesOcupacaoGeral($row)
    {   
        $totais = array();
        $row['NRESTRUTLEGAL'] = null;
        $row['NRESTRUTGEREN'] = null;
        $row = $this->verificaPerfilAcesso($row);
        $row = $this->adicionaCondicao($row);


        $totais = $this->indicadoresSegurancaTrabalhoRepository->getTotalAcidentesOcupacaoGeral(
            $row
        );

        return $totais;
    }

    public function getTotalAcidentesPorGravidadeLegalOcupacao($row)
    {   
        $totais = array();
        $row['NRESTRUTLEGAL'] = null;
        $row['NRESTRUTGEREN'] = null;
        $row = $this->verificaPerfilAcesso($row);
        $row = $this->adicionaCondicao($row);

        $totais = $this->indicadoresSegurancaTrabalhoRepository->getTotalAcidentesPorGravidadeLegalOcupacao(
            $row
        );

        return $totais;
    }

    public function getTotalAcidentesPorGravidadeGerenOcup($row)
    {   

        $totais = array();
        $row['NRESTRUTLEGAL'] = null;
        $row['NRESTRUTGEREN'] = null;
        $row = $this->verificaPerfilAcesso($row);
        $row = $this->adicionaCondicao($row);

        $totais = $this->indicadoresSegurancaTrabalhoRepository->getTotalAcidentesPorGravidadeGerenOcup(
            $row
        );

        return $totais;
    }

    public function getTotalAcidentesPorGravidadeOcupacaoGeral($row)
    {   
        $totais = array();
        $row['NRESTRUTLEGAL'] = null;
        $row['NRESTRUTGEREN'] = null;
        $row = $this->verificaPerfilAcesso($row);
        $row = $this->adicionaCondicao($row);

        $totais = $this->indicadoresSegurancaTrabalhoRepository->getTotalAcidentesPorGravidadeOcupacaoGeral(
            $row
        );

        return $totais;
    }

    public function getQuantidadeDiasAfastadosLegalOcup($row)
    {   
        $totais = array();
        $row['NRESTRUTLEGAL'] = null;
        $row['NRESTRUTGEREN'] = null;
        $row = $this->verificaPerfilAcesso($row);
        $row = $this->adicionaCondicao($row);

        $totais = $this->indicadoresSegurancaTrabalhoRepository->getQuantidadeDiasAfastadosLegalOcup(
            $row
        );

        return $totais;
    }

    public function getQuantidadeDiasAfastadosOcupacaoGeral($row)
    {   
        $totais = array();
        $row['NRESTRUTLEGAL'] = null;
        $row['NRESTRUTGEREN'] = null;
        $row = $this->verificaPerfilAcesso($row);
        $row = $this->adicionaCondicao($row);

        $totais = $this->indicadoresSegurancaTrabalhoRepository->getQuantidadeDiasAfastadosOcupacaoGeral(
            $row
        );

        return $totais;
    }

    public function getTotalPartesDoCorpoAtingidasLegalOcup($row)
    {   
        $totais = array();
        $row['NRESTRUTLEGAL'] = null;
        $row['NRESTRUTGEREN'] = null;
        $row = $this->verificaPerfilAcesso($row);
        $row = $this->adicionaCondicao($row);

        $totais = $this->indicadoresSegurancaTrabalhoRepository->getTotalPartesDoCorpoAtingidasLegalOcup(
            $row
        );

        return $totais;
    }

    public function getTotalPartesDoCorpoAtingidasOcupacaoGeral($row)
    {   
        $totais = array();
        $row['NRESTRUTLEGAL'] = null;
        $row['NRESTRUTGEREN'] = null;
        $row = $this->verificaPerfilAcesso($row);
        $row = $this->adicionaCondicao($row);

        $totais = $this->indicadoresSegurancaTrabalhoRepository->getTotalPartesDoCorpoAtingidasOcupacaoGeral(
            $row
        );

        return $totais;
    }

    public function getSituacaoGeradoraAcidenteLegalOcup($row)
    {   
        $totais = array();
        $row['NRESTRUTLEGAL'] = null;
        $row['NRESTRUTGEREN'] = null;
        $row = $this->verificaPerfilAcesso($row);
        $row = $this->adicionaCondicao($row);

        $totais = $this->indicadoresSegurancaTrabalhoRepository->getSituacaoGeradoraAcidenteLegalOcup(
            $row
        );

        return $totais;
    }

    public function getSituacaoGeradoraAcidenteOcupacaoGeral($row)
    {   
        $totais = array();
        $row['NRESTRUTLEGAL'] = null;
        $row['NRESTRUTGEREN'] = null;
        $row = $this->verificaPerfilAcesso($row);
        $row = $this->adicionaCondicao($row);

        $totais = $this->indicadoresSegurancaTrabalhoRepository->getSituacaoGeradoraAcidenteOcupacaoGeral(
            $row
        );

        return $totais;
    }

    public function getTotalEpisEntreguesLegalOcup($row)
    {   
        $totais = array();
        $row['NRESTRUTLEGAL'] = null;
        $row['NRESTRUTGEREN'] = null;
        $row = $this->verificaPerfilAcesso($row);
        $row = $this->adicionaCondicao($row);


        if(!isset($row['DTMESCOMPETENC'])) {
            $row['DTMESCOMPETENC'] = Utils::getParamters()->getDtmescomptencDiaCorrente();

            $totais = $this->indicadoresSegurancaTrabalhoRepository->getTotalEpisEntreguesLegalOcupDashboard(
                $row
            );
            
            return $totais;
        }

        $totais = $this->indicadoresSegurancaTrabalhoRepository->getTotalEpisEntreguesLegalOcup(
            $row
        );

        return $totais;
    }

    public function getTotalEpisEntreguesGerenOcup($row)
    {   
        $totais = array();
        $row['NRESTRUTLEGAL'] = null;
        $row['NRESTRUTGEREN'] = null;
        $row = $this->verificaPerfilAcesso($row);
        $row = $this->adicionaCondicao($row);

        $totais = $this->indicadoresSegurancaTrabalhoRepository->getTotalEpisEntreguesGerenOcup(
            $row
        );

        return $totais;
    }

    public function getTotalEpisEntreguesOcupacaoGeral($row)
    {   
        $totais = array();
        $row['NRESTRUTLEGAL'] = null;
        $row['NRESTRUTGEREN'] = null;
        $row = $this->verificaPerfilAcesso($row);
        $row = $this->adicionaCondicao($row);

        $totais = $this->indicadoresSegurancaTrabalhoRepository->getTotalEpisEntreguesOcupacaoGeral(
            $row
        );

        return $totais;
    }

    public function getTotalEpisEntreguesVencidoLegalOcupacao($row)
    {   
        $totais = array();
        $row['NRESTRUTLEGAL'] = null;
        $row['NRESTRUTGEREN'] = null;
        $row = $this->verificaPerfilAcesso($row);
        $row = $this->adicionaCondicao($row);

        $totais = $this->indicadoresSegurancaTrabalhoRepository->getTotalEpisEntreguesVencidoLegalOcupacao(
            $row
        );

        return $totais;
    }

    public function getTotalEpisEntreguesVencidoGerenOcup($row)
    {   
        $totais = array();
        $row['NRESTRUTLEGAL'] = null;
        $row['NRESTRUTGEREN'] = null;
        $row = $this->verificaPerfilAcesso($row);
        $row = $this->adicionaCondicao($row);

        $totais = $this->indicadoresSegurancaTrabalhoRepository->getTotalEpisEntreguesVencidoGerenOcup(
            $row
        );

        return $totais;
    }

    public function getTotalEpisEntreguesVencidoOcupacaoGeral($row)
    {   
        $totais = array();
        $row['NRESTRUTLEGAL'] = null;
        $row['NRESTRUTGEREN'] = null;
        $row = $this->verificaPerfilAcesso($row);
        $row = $this->adicionaCondicao($row);

        $totais = $this->indicadoresSegurancaTrabalhoRepository->getTotalEpisEntreguesVencidoOcupacaoGeral(
            $row
        );

        return $totais;
    }

    public function getTotalEpisEntreguesCAVencidoLegalOcup($row)
    {   
        $totais = array();
        $row['NRESTRUTLEGAL'] = null;
        $row['NRESTRUTGEREN'] = null;
        $row = $this->verificaPerfilAcesso($row);
        $row = $this->adicionaCondicao($row);

        $totais = $this->indicadoresSegurancaTrabalhoRepository->getTotalEpisEntreguesCAVencidoLegalOcup(
            $row
        );

        return $totais;
    }

    public function getTotalEpisEntreguesCAVencidoGerenOcup($row)
    {   
        $totais = array();
        $row['NRESTRUTLEGAL'] = null;
        $row['NRESTRUTGEREN'] = null;
        $row = $this->verificaPerfilAcesso($row);
        $row = $this->adicionaCondicao($row);

        $totais = $this->indicadoresSegurancaTrabalhoRepository->getTotalEpisEntreguesCAVencidoGerenOcup(
            $row
        );

        return $totais;
    }

    public function getTotalEpisEntreguesCAVencidoOcupacaoGeral($row)
    {   
        $totais = array();
        $row['NRESTRUTLEGAL'] = null;
        $row['NRESTRUTGEREN'] = null;
        $row = $this->verificaPerfilAcesso($row);
        $row = $this->adicionaCondicao($row);

        $totais = $this->indicadoresSegurancaTrabalhoRepository->getTotalEpisEntreguesCAVencidoOcupacaoGeral(
            $row
        );

        return $totais;
    }

    public function getReuniaoPorMes($row)
    {   
        $totais = array();
        $row['NRESTRUTLEGAL'] = null;
        $row['NRESTRUTGEREN'] = null;
        $row = $this->verificaPerfilAcesso($row);
        $row = $this->adicionaCondicao($row);

        $totais = $this->indicadoresSegurancaTrabalhoRepository->getReuniaoPorMes(
            $row
        );

        return $totais;
    }

    public function getReuniaoPorAno($row)
    {   
        $totais = array();
        $row['NRESTRUTLEGAL'] = null;
        $row['NRESTRUTGEREN'] = null;
        $row = $this->verificaPerfilAcesso($row);
        $row = $this->adicionaCondicao($row);

        if(!isset($row['DTMESCOMPETENC'])) {
            $row['DTMESCOMPETENC'] = Utils::getParamters()->getDtmescomptencDiaCorrente();

            $totais = $this->indicadoresSegurancaTrabalhoRepository->getTotalReuniaoAnoDashboard(
                $row
            );
            
            return $totais;
        }

        $totais = $this->indicadoresSegurancaTrabalhoRepository->getReuniaoPorAno(
            $row
        );

        return $totais;
    }

    public function getTotalReuniaoPorTipoReuniao($row)
    {   
        $totais = array();
        $row['NRESTRUTLEGAL'] = null;
        $row['NRESTRUTGEREN'] = null;
        $row = $this->verificaPerfilAcesso($row);
        $row = $this->adicionaCondicao($row);

        $totais = $this->indicadoresSegurancaTrabalhoRepository->getTotalReuniaoPorTipoReuniao(
            $row
        );

        return $totais;
    }
}