<?php
namespace Folha\Logic\ControlePonto;

use Folha\Entities\GpeEscalatrabm;
use Folha\Entities\GpeEscalatrabh;
use Folha\Entities\GpeAlteescala;
use Folha\Entities\GpeMarcacaoponto;
use Folha\Entities\GpeVinculom;
use Folha\Entities\PtoIntervalosmarcacao;
use Folha\Entities\PtoResumositufreq;
use Folha\Entities\PtoSitufrequencia;
use Folha\Logic\ControlePonto\DateUtil;
use Folha\Logic\ControlePonto\Dia;
use Folha\Logic\ControlePonto\FPA;
/**
 * Apuracao.
 *
 * @author  ingridmoreira
 */

class Apuracao{

    private $nrorg;
    private $nrvinculom;
    private $iniPeriodo;
    private $fimPeriodo;
    private $nrPeriododApuracao;
    private $periodo = array();
    private $datasInvalidas;

    private $dtApuracaoDiaria = null;

    //Dia Ocorrência
    const DIA_ANTERIOR       = 'DA';
    const DIA_CORRENTE       = 'DC';
    const DIA_SEGUINTE       = 'DS';
    const FORCA_DIA_ANTERIOR = 'FA';
    const FORCA_DIA_CORRENTE = 'FC';
    const FORCA_DIA_SEGUINTE = 'FS';



    public function toJson(){
        reset($this->periodo);
        $arrayJson = array();
        do{
            /** @var Dia $dia */
            $dia = current($this->periodo);
            if(is_object($dia)) {

                /** @var PtoResumositufreq $resumoSituFreq */
                foreach ($dia->getResumosSituFreqs() as $resumoSituFreq) {
                    /** @var PtoSitufrequencia $situFrequencia */
                    $situFrequencia = PtoSitufrequencia::findOneByNrsitufrequencia($resumoSituFreq->getNrsitufrequencia());
                    if (is_object($situFrequencia)) {
                        $advertSituFreq = $situFrequencia->getAdvertsitufreq();
                        if (is_object($advertSituFreq)) {
                            $arrayJson['corAdvertSituFreq'] = $advertSituFreq->getNrcoradvertsitufreq();
                            break;
                        }
                    }
                }
            }

        }while(next($this->periodo));
        $arrayJson['qtdDatasInvalidas'] = count($this->datasInvalidas);
        return $arrayJson;
    }
    /**
     * Construtor.
     *
     * @author ingridmoreira
     */
    public function __construct($nrorg, $nrvinculom, $dtIniPeriodo, $dtFimPeriodo, $nrPeriododApuracao, $dtApuracaoDiaria = null){
        // set_time_limit(0);
        
        $this->nrorg = $nrorg;
        $this->nrvinculom = $nrvinculom;
        $this->iniPeriodo = DateUtil::getDataDeString($dtIniPeriodo,
                                                      DateUtil::FORMATO_BRASILEIRO,
                                                      true);
        $this->fimPeriodo = DateUtil::getDataDeString($dtFimPeriodo,
                                                      DateUtil::FORMATO_BRASILEIRO,
                                                      true);
        $this->nrPeriododApuracao = $nrPeriododApuracao;
        $fimPeriodoAux = DateUtil::adicionaIntervalo($this->fimPeriodo, 1, DateUtil::DIA);
        $data = $this->iniPeriodo;
        $periodo = array();
        $datasInvalidas = array();
        $fromAplicativo = !is_null($dtApuracaoDiaria);
        $this->dtApuracaoDiaria = $dtApuracaoDiaria;
        //Definição dos dados do período a ser apurado
        /* Vitor Peixoto 25/05/2015
         * Array externo passado por referência para manter
         * valores entre os dias
         */
        $refAuxSaldos = array('HORAS_FOLGAR' => array('DIA' => 0,
                                                      'PER' => 0,
                                                      'BDH' => 0,
                                                      'SOMOU_SALDO_BDH_ANTIGO' => false),

                              'HORAS_FOLGAR_ACRESC' => array('DIA' => 0,
                                                         'PER' => 0,
                                                         'BDH' => 0,
                                                         'SOMOU_SALDO_BDH_ANTIGO' => false),
                              'DSR_DESCONTADO' => [],
                              'GEROU_DSR' => false,
                              'FERIADO' => false
        );

        /* END Vitor Peixoto*/

        while (DateUtil::comparaDatas($data, $fimPeriodoAux, DateUtil::ANTES) ||
            DateUtil::comparaDatas($data, $fimPeriodoAux, DateUtil::IGUAL)) {
            $dataString = $data->format(DateUtil::FORMATO_BRASILEIRO);
            $where = "NRORG = ?
               AND NRVINCULOM = ?
               AND TO_DATE(?,'DD/MM/YYYY')
               BETWEEN DTINIESCALA
               AND NVL(DTFIMESCALA, TO_DATE(?,'DD/MM/YYYY'))";
            $params = array(
                $this->nrorg,
                $this->nrvinculom,
                $dataString,
                $dataString
            );
            $orderBy = "DTINIESCALA DESC";
            /** @var GpeAlteescala $alteEscala */
            $alteEscala = GpeAlteescala::getObjectByWhere($where, $params, $orderBy);
            
            $dentroPeriodo = true;
            if (DateUtil::comparaDatas($data, $this->fimPeriodo, DateUtil::DEPOIS)) {
                $dentroPeriodo = false;
            }
            if (is_object($alteEscala) && (is_null($dtApuracaoDiaria) || DateUtil::comparaDatas(DateUtil::subtraiIntervalo($data, 1), $dtApuracaoDiaria, DateUtil::IGUAL)
                                                                      || DateUtil::comparaDatas($data, $dtApuracaoDiaria, DateUtil::IGUAL))) {
                                                                          
                // Modificação | viniciusmayrink
                // 31/01/2017 - leonardocarvalho - valida parametrização de tempo de descanso diário
                \Folha\Logic\ControlePonto\ControlePonto::validaDescansoDiario($data->format(DateUtil::FORMATO_BRASILEIRO), $this->nrvinculom);

                // 20/12/2017 - leonardocarvalho - valida existência de ocorrências pendentes para o dia
                \Folha\Logic\ControlePonto\ControlePonto::validaOcorrenciasPendentes($data->format(DateUtil::FORMATO_BRASILEIRO), $this->nrvinculom);

                // 21/06/2024 - richbertoliveira - valida existência de ocorrências rejeitadas para o dia
                \Folha\Logic\ControlePonto\ControlePonto::validaOcorrenciasRejeitadas($data->format(DateUtil::FORMATO_BRASILEIRO), $this->nrvinculom);

                $escala = GpeEscalatrabm::findOneByNrescalatrabmAndNrorg($alteEscala->getNrescalatrabm(), $this->nrorg);
                $nrHorDiaM = $alteEscala->getNrhordiam();

                $alteracaoEscalaNaoControlaJornada = $alteEscala->getIdcontjornada() == 'S';
                $historicoEscala = $this->getHistoricoEscala($alteEscala->getNrescalatrabm());
                $historicoEscalaNaoControlaJornada = !empty($historicoEscala) ? $historicoEscala->getIdnaocontrolajornada() == 'S' : false;

                $controlaJornada = !$alteracaoEscalaNaoControlaJornada && !$historicoEscalaNaoControlaJornada;

                // modificação | viniciusmayrink
                // if(!is_null($alteEscala->getQthrlimitesephextra()) && $alteEscala->getQthrlimitesephextra() != '') {
                    $escala->Qtlimsephrextra = $alteEscala->getQthrlimitesephextra();
                // } 

                if (is_object($escala) && $controlaJornada) {
                    $turno = $alteEscala->getTurno();
                    
                    if (is_object($turno)) {
                        /* Vitor Peixoto 26/05 */
                            $refAuxSaldos['HORAS_FOLGAR']['DIA']        = 0;
                            $refAuxSaldos['HORAS_FOLGAR_ACRESC']['DIA'] = 0;
                        /* END Vitor Peixoto */
                        $dataAuxProx = DateUtil::adicionaIntervalo($data, 1, DateUtil::DIA)->format(DateUtil::FORMATO_BRASILEIRO);
                        $where = "NRORG = ?
                               AND NRVINCULOM = ?
                               AND TO_DATE(?,'DD/MM/YYYY')
                               BETWEEN DTINIESCALA
                               AND NVL(DTFIMESCALA, TO_DATE(?,'DD/MM/YYYY'))";
                        $params = array(
                            $this->nrorg,
                            $this->nrvinculom,
                            $dataAuxProx,
                            $dataAuxProx
                        );
                        $orderBy = "DTINIESCALA DESC";
                        $alteEscalaProx = GpeAlteescala::getObjectByWhere($where, $params, $orderBy);
                        
                        if (is_object($alteEscalaProx)) {
                            $escalaProx = GpeEscalatrabm::findOneByNrescalatrabmAndNrorg($alteEscalaProx->getNrescalatrabm(),
                                                                                         $this->nrorg);
                            $turnoProx = $alteEscalaProx->getTurno();
                            
                            if (is_object($escalaProx)) {
                                if (is_object($turnoProx)) {
                                    $periodo[] = new Dia($this->nrorg, $this->nrvinculom, $data,
                                                         $escala, $turno, $dentroPeriodo, $this->nrPeriododApuracao,
                                                         $nrHorDiaM, $this->iniPeriodo, $fromAplicativo);
                                }
                                elseif ($dentroPeriodo == true) {
                                    $datasInvalidas['DIASEGUINTE'][] = $dataString;
                                    $periodo[] = new Dia($this->nrorg, $this->nrvinculom, $data,
                                                         $escala, $turno, $dentroPeriodo, $this->nrPeriododApuracao,
                                                         $nrHorDiaM, $this->iniPeriodo, $fromAplicativo);
                                }
                                else{
                                    $periodo[] = new Dia($this->nrorg, $this->nrvinculom, $data,
                                        $escala, $turno, $dentroPeriodo, $this->nrPeriododApuracao,
                                        $nrHorDiaM, $this->iniPeriodo, $fromAplicativo);
                                }
                            }
                            elseif ($dentroPeriodo == true) {
                                $datasInvalidas['DIASEGUINTE'][] = $dataString;
                                $periodo[] = new Dia($this->nrorg, $this->nrvinculom, $data,
                                                     $escala, $turno, $dentroPeriodo, $this->nrPeriododApuracao,
                                                     $nrHorDiaM, $this->iniPeriodo, $fromAplicativo);
                            }
                            else{
                                $periodo[] = new Dia($this->nrorg, $this->nrvinculom, $data,
                                    $escala, $turno, $dentroPeriodo, $this->nrPeriododApuracao,
                                    $nrHorDiaM, $this->iniPeriodo, $fromAplicativo);
                            }
                        }
                        elseif ($dentroPeriodo == true) {
                            $datasInvalidas['DIASEGUINTE'][] = $dataString;
                            $periodo[] = new Dia($this->nrorg, $this->nrvinculom, $data,
                                                 $escala, $turno, $dentroPeriodo, $this->nrPeriododApuracao,
                                                 $nrHorDiaM, $this->iniPeriodo, $fromAplicativo);
                        }
                        else{
                            $periodo[] = new Dia($this->nrorg, $this->nrvinculom, $data,
                                $escala, $turno, $dentroPeriodo, $this->nrPeriododApuracao,
                                $nrHorDiaM, $this->iniPeriodo, $fromAplicativo);
                        }
                    }
                    elseif ($dentroPeriodo == true) {
                        $datasInvalidas['TURNOS'][] = $dataString;
                    }
                }
                elseif ($dentroPeriodo == true) {
                    $datasInvalidas['ESCALAS'][] = $dataString;
                }
            }
            elseif ($dentroPeriodo == true) {
                $datasInvalidas['ALTEESCALA'][] = $dataString;
            }

            $data = DateUtil::adicionaIntervalo($data, 1, DateUtil::DIA);
        }
        $this->datasInvalidas = $datasInvalidas;

        /** @var $dia \Folha\Logic\ControlePonto\Dia */
        foreach ($periodo as $indice => &$dia) {
            $dataAux = $dia->getData()->format(DateUtil::FORMATO_BRASILEIRO);
            
            $invalida = false;
            if(isset($this->datasInvalidas['DIASEGUINTE'])) {
                $invalida = in_array($dataAux, $this->datasInvalidas['DIASEGUINTE']);
            }
            if ($dia->getDentroPeriodo() && !$invalida) {
                $proximoDia = $periodo[$indice + 1] ?? null;
                $horarioProximoDia = !empty($proximoDia) ? $proximoDia->getHorario() : null;
                $idDiaCalendProximoDia = !empty($proximoDia) ? $proximoDia->getIdDiaCalend() : null;

                $dia->setRegistrosDia(
                    $horarioProximoDia,
                    $idDiaCalendProximoDia,
                    $refAuxSaldos,
                    $proximoDia,
                    $periodo
                );
            }
        }
        
        $this->periodo = $periodo;
        gc_enable();
    }

    public function getNrorg(){
        return $this->nrorg;
    }
    
    public function setNrorg($nrorg){
        $this->nrorg = $nrorg;
    }
    
    public function getNrvinculom(){
        return $this->nrvinculom;
    }
    
    public function setNrvinculom($nrvinculom){
        $this->nrvinculom = $nrvinculom;
    }
    
    public function getIniPeriodo(){
        return $this->iniPeriodo;
    }
    
    public function setIniPeriodo($iniPeriodo){
        $this->iniPeriodo = $iniPeriodo;
    }
    
    public function getFimPeriodo(){
        return $this->fimPeriodo;
    }
    
    public function setFimPeriodo($fimPeriodo){
        $this->fimPeriodo = $fimPeriodo;
    }
    
    public function getPeriodo(){
        return $this->periodo;
    }
    
    public function setPeriodo($periodo){
        $this->periodo = $periodo;
    }
    
    public function getDatasInvalidas(){
        return $this->datasInvalidas;
    }
    
    public function setDatasInvalidas($datasInvalidas){
        $this->datasInvalidas = $datasInvalidas;
    }
    
    public function getDiaPeriodo($data){
        foreach($this->periodo as $key => $dia){
            
            if(DateUtil::comparaDatas($data, $dia->getData(), DateUtil::IGUAL)){
                return array('CHAVE' => $key,
                             'DIA'   => $dia);
            }
        }   
    }

    public function salvaAlterRegistros($linhasModificadas, $dtDiaSelecionado){
        $periodo             = $this->periodo;
        $dadosDiaSelecionado = self::getDiaPeriodo($dtDiaSelecionado);
        /** @var Dia $diaSelecionado  */
        $diaSelecionado      = $dadosDiaSelecionado['DIA'];

        foreach($linhasModificadas as $linhaModificada){

            $db = \Setforms2\Shared\Registry::getEntityManager()->beginTransaction();
            $connection = \Setforms2\Shared\Registry::getEntityManagerConnection();
            /** @var $marcacao \Folha\Entities\GpeMarcacaoponto */
            $marcacao = GpeMarcacaoponto::findOneByNrorgAndNrmarcacaoponto($linhaModificada['NRORG'],
                                                                           $linhaModificada['NRMARCACAOPONTO']);
            try{
                $marcacao->setNrorgultatu(FPA::getNrorg());
                $marcacao->setDtultatu(DateUtil::getDataAtual());
                $marcacao->setCdoperultatu(FPA::getCdOperador());
                $marcacao->setIddiaocorrencia($linhaModificada['IDDIAOCORRENCIA']);
                $marcacao->setIdmarcdesprezada($linhaModificada['IDMARCDESPREZADA']);

                $marcacao->persist();

                $connection->flush();

                $db->getConn()->commit();

            }catch(\Exception $e){
                $db->getConn()->rollback();
                return false;
            }

            $dtHrLinhaMod = DateUtil::getDataDeString($linhaModificada['DTHRAJUSTADA'], 'd/m/Y H:i', true);
            $dtHrAjustada = DateUtil::getDataDeString($linhaModificada['DTHRAJUSTADA'], 'd/m/Y H:i');
            $dtStLinhaMod = $dtHrLinhaMod->format(DateUtil::FORMATO_BRASILEIRO);

            //Datas/Horas de início e fim de coleta
            $dadosDiaLinhaMod = self::getDiaPeriodo($dtHrLinhaMod);
            /** @var $diaLinhaMod \Folha\Logic\ControlePonto\Dia */
            $diaLinhaMod   = $dadosDiaLinhaMod['DIA'];
            /** @var $diaSeguinte \Folha\Logic\ControlePonto\Dia */
            $diaSeguinte = $periodo[$dadosDiaLinhaMod['CHAVE']+1];
            $dtStDiaSeg  = $diaSeguinte->getData()->format(DateUtil::FORMATO_BRASILEIRO);

            $dtHrIniColeta = $dtStLinhaMod.' '.self::formataHora($diaLinhaMod->getHorario()->getHorarioHistorico()->getHrinicoletamarcacao());
            $dtHrIniColeta = DateUtil::getDataDeString($dtHrIniColeta, 'd/m/Y H:i');

            $horarioFimColeta = self::formataHora($diaSeguinte->getHorario()->getHorarioHistorico()->getHrinicoletamarcacao());
            $horarioFimColeta = DateUtil::subtrairHoras($horarioFimColeta, '00:01' , DateUtil::HORA_MINUTO);

            $dtHrFimColeta = $dtStDiaSeg.' '.$horarioFimColeta;
            $dtHrFimColeta = DateUtil::getDataDeString($dtHrFimColeta, 'd/m/Y H:i');

            if(($linhaModificada['IDDIAOCORRENCIA'] == self::DIA_ANTERIOR ||
                $linhaModificada['IDDIAOCORRENCIA'] == self::DIA_SEGUINTE ||
                $linhaModificada['IDDIAOCORRENCIA'] == self::DIA_CORRENTE)){
                if(DateUtil::comparaDatas($dtHrAjustada, $dtHrIniColeta, DateUtil::ANTES)){
                    if(DateUtil::comparaDatas($dtHrLinhaMod, $dtDiaSelecionado, DateUtil::IGUAL)){
                        $keyDiaMod = $dadosDiaSelecionado['CHAVE'] - 1;
                    }
                    elseif(DateUtil::comparaDatas($dtHrLinhaMod, $dtDiaSelecionado, DateUtil::ANTES)){
                        $keyDiaMod = $dadosDiaSelecionado['CHAVE'] - 2;
                    }
                    else{
                        $keyDiaMod = $dadosDiaSelecionado['CHAVE'];
                    }
                }
                elseif(!DateUtil::comparaDatas($dtHrAjustada, $dtHrIniColeta, DateUtil::ANTES) &&
                        !DateUtil::comparaDatas($dtHrAjustada, $dtHrFimColeta, DateUtil::DEPOIS)){
                    if(DateUtil::comparaDatas($dtHrLinhaMod, $dtDiaSelecionado, DateUtil::DEPOIS)){
                        $keyDiaMod = $dadosDiaSelecionado['CHAVE'] + 1;
                    }
                    elseif(DateUtil::comparaDatas($dtHrLinhaMod, $dtDiaSelecionado, DateUtil::ANTES)){
                        $keyDiaMod = $dadosDiaSelecionado['CHAVE'] - 1;
                    }
                    else{
                        $keyDiaMod = $dadosDiaSelecionado['CHAVE'];
                    }
                }
                else{
                    if(DateUtil::comparaDatas($dtHrLinhaMod, $dtDiaSelecionado, DateUtil::IGUAL)){
                        $keyDiaMod = $dadosDiaSelecionado['CHAVE'] + 1;
                    }
                    elseif(DateUtil::comparaDatas($dtHrLinhaMod, $dtDiaSelecionado, DateUtil::ANTES)){
                        $keyDiaMod = $dadosDiaSelecionado['CHAVE'] + 2;
                    }
                    else{
                        $keyDiaMod = $dadosDiaSelecionado['CHAVE'];
                    }
                }
            }
            elseif($linhaModificada['IDDIAOCORRENCIA'] == self::FORCA_DIA_ANTERIOR){
                if(DateUtil::comparaDatas($dtHrLinhaMod, $dtDiaSelecionado, DateUtil::IGUAL)){
                    $keyDiaMod = $dadosDiaSelecionado['CHAVE'] - 1;
                }
                elseif(DateUtil::comparaDatas($dtHrLinhaMod, $dtDiaSelecionado, DateUtil::ANTES)){
                    $keyDiaMod = $dadosDiaSelecionado['CHAVE'] - 2;
                }
                else{
                    $keyDiaMod = $dadosDiaSelecionado['CHAVE'];
                }
            }
            elseif($linhaModificada['IDDIAOCORRENCIA'] == self::FORCA_DIA_SEGUINTE){
                if(DateUtil::comparaDatas($dtHrLinhaMod, $dtDiaSelecionado, DateUtil::IGUAL)){
                    $keyDiaMod = $dadosDiaSelecionado['CHAVE'] + 1;
                }
                elseif(DateUtil::comparaDatas($dtHrLinhaMod, $dtDiaSelecionado, DateUtil::DEPOIS)){
                    $keyDiaMod = $dadosDiaSelecionado['CHAVE'] + 2;
                }
                else{
                    $keyDiaMod = $dadosDiaSelecionado['CHAVE'];
                }
            }
            else{
                if(DateUtil::comparaDatas($dtHrLinhaMod, $dtDiaSelecionado, DateUtil::DEPOIS)){
                    $keyDiaMod = $dadosDiaSelecionado['CHAVE'] + 1;
                }
                elseif(DateUtil::comparaDatas($dtHrLinhaMod, $dtDiaSelecionado, DateUtil::ANTES)){
                    $keyDiaMod = $dadosDiaSelecionado['CHAVE'] - 1;
                }
                else{
                    $keyDiaMod = $dadosDiaSelecionado['CHAVE'];
                }
            }

            $countDias = 0;
            /** @var $dia Dia */
            foreach($periodo as $indice => &$dia){
                $invalida = false;
                if(isset($this->datasInvalidas['DIASEGUINTE'])) {
                    $invalida = in_array($dia->getData()->format(DateUtil::FORMATO_BRASILEIRO), $this->datasInvalidas['DIASEGUINTE']);
                }
                if($dia->getDentroPeriodo() && !$invalida && ($indice == $keyDiaMod || $indice == $dadosDiaSelecionado['CHAVE'])){
                    $countDias++;
                    $dia->limpaResumosNaoForcados();
                    $dia->setRegistrosDia($periodo[$indice+1]->getHorario(), $periodo[$indice+1]->getIdDiaCalend());
                }
                if($countDias == 2){
                    break;
                }
            }
        }

        $this->periodo = $periodo;

        return true;
    }

    public static function buscaApuracao($nrOrg, $nrApuraFrequencia){

        $apuracao   = \Folha\Entities\GpeApurafrequencia::findOneByNrorgAndNrapurafrequencia($nrOrg, $nrApuraFrequencia);

        return $apuracao;
    }

    private function formataHora($hora)
    {
        $hora = DateUtil::getHoraFloatDeString($hora);
        $hora = DateUtil::getHoraStringDeFloat($hora, true);

        return $hora;
    }

    public function gravaApuracao($merge = false, $retrocedeDia = false/*, $gravarMarcacoes = true*/)
    {
        if($retrocedeDia) {
            \Windows\WinPto00005000\TabPto00005000000\Facade::retrocessoDiario($this->nrorg, $this->nrvinculom, $this->dtApuracaoDiaria);
        }
        else {
            reset($this->periodo);
            $operador = FPA::getCdOperador();
            $nrOrg = FPA::getNrorg();
            $dataAtual = DateUtil::getDataAtual();
            $db = \Setforms2\Shared\Registry::getEntityManager();
            $db->beginTransaction();

            try {
                do {
                    /** @var Dia $dia */
                    $dia = current($this->periodo);
                    if (is_object($dia)) {
                        
                        // Modificação | viniciusmayrink
                        $novosRegistros = self::gravaMarcacoes($dia->getRegistros(), $dataAtual, $nrOrg, $operador, $merge);
                        $dia->setRegistros($novosRegistros);
                        unset($novosRegistros); // Marca a variável para liberação

                        $dia = self::organizaIntervalosComNumerosMarcacao($dia);
                        $novosIntervalos = self::gravaIntervalosMarcacao($dia->getIntervalosRegistros(), $dataAtual, $nrOrg, $operador);
                        
                        $dia->setIntervalosRegistros($novosIntervalos);
                        unset($novosIntervalos); // Marca a variável para liberação

                        /* $novosResumos = */ //Alteração Vitor Peixoto 20/07/2015 Economia de variável inutilizada
                        self::gravaResumosSituFreq($dia->getResumosSituFreqs(), $dataAtual, $nrOrg, $operador, $merge);
                        //$dia->setResumosSituFreqs($novosResumos);

                        unset($dia);         // Marca a variável para liberação
                        gc_collect_cycles(); // Invoca o Garbage Colector para liberar a memória
                    }
                } while (next($this->periodo));
                return $db->commit();

            } catch (\Exception $ex) {
                $db->rollback();
                return $ex->getMessage();
            }
        }
    }

    private function gravaResumosSituFreq($resumosSituFreq, $dtAtual, $nrOrg, $operador){
        $entityManager = \Setforms2\Shared\Registry::getEntityManager();
        /** @var PtoResumositufreq $resumoSituFreq */

        try {
            foreach ($resumosSituFreq as &$resumoSituFreq) {
                $nrResumoSituFreq = \Geral\Util\NovoCodigo::geraCodigo($resumoSituFreq->getNrorg(), 'PTO_RESUMOSITUFREQ', 1, 10, true, false);
                //Força erro pra teste
//            if($this->nrvinculom == 1339){
//                $nrResumoSituFreq = null;
//            }
                if(empty($resumoSituFreq->getNrmotivoocorfreq()) || $resumoSituFreq->getNrmotivoocorfreq() === '') {
                    $resumoSituFreq->setNrmotivoocorfreq(null);
                }
                
                $resumoSituFreq->setNrresumositufreq($nrResumoSituFreq);
                $resumoSituFreq->setDtinclusao($dtAtual);
                $resumoSituFreq->setNrorginclusao($nrOrg);
                $resumoSituFreq->setCdoperinclusao($operador);
                $resumoSituFreq->setDtultatu($dtAtual);
                $resumoSituFreq->setNrorgultatu($nrOrg);
                $resumoSituFreq->setCdoperultatu($operador);
                $resumoSituFreq->persist();
            }
            unset($resumosSituFreq);
            $entityManager->flush(); //Sincroniza com o banco
            $entityManager->clear(); //Limpa os dados do EntityManager
            gc_collect_cycles();     //Invoca o Garbage Colector
        } catch (\Exception $e) {
            //\Setforms2\Shared\Registry::createManager();
            echo $e->getMessage();
            throw $e;
        }
        //return $resumosSituFreq;
    }

    private function gravaMarcacoes($registrosMarcacao, $dtAtual, $nrOrg, $operador, $merge){
        $marcacao = null;
        $entityManager = \Setforms2\Shared\Registry::getEntityManager();
        $entityManager->getConnection()->getConfiguration()->setSQLLogger(null);
        
        try {
            /** @var RegistroPonto $registroMarcacao */
            foreach ($registrosMarcacao as &$registroMarcacao) {
                $marcacao = $registroMarcacao->getMarcacao();
                if (!$marcacao->getNrmarcacaoponto()) {
                    $nrMarcacaoPonto = \Geral\Util\NovoCodigo::geraCodigo($marcacao->getNrorg(), 'GPE_MARCACAOPONTO', 1, 10, true, false);
                    $marcacao->setNrmarcacaoponto($nrMarcacaoPonto);
                    $marcacao->setDtinclusao($dtAtual);
                    $marcacao->setNrorginclusao($nrOrg);
                    $marcacao->setCdoperinclusao($operador);
                } else {
                    $marcacao = $entityManager->merge($marcacao);
                }

                $marcacao->setDtultatu($dtAtual);
                $marcacao->setNrorgultatu($nrOrg);
                $marcacao->setCdoperultatu($operador);

                $registroMarcacao->setMarcacao($marcacao);
                $marcacao->persist();
                unset($marcacao); // Marca a variável para liberação
            }
            $entityManager->flush(); //Sincroniza com o banco
            $entityManager->clear(); //Limpa os dados do EntityManager
            gc_collect_cycles();     //Invoca o Garbage Colector
        } catch (\Exception $e) {
            //\Setforms2\Shared\Registry::createManager();
            throw new \Exception('Problema na inserção da marcação de vínculo '.$this->nrvinculom . " Error: ". $e->getMessage()/*$marcacao->getDthrajustada()*/);
        }

        return $registrosMarcacao;
    }

    private function gravaIntervalosMarcacao($intervalosMarcacao, $dtAtual, $nrOrg, $operador){        
        try {
            /** @var PtoIntervalosmarcacao $intervaloMarcacao */
            foreach ($intervalosMarcacao as &$intervaloMarcacao) {

                if (is_null($intervaloMarcacao->getNrintervalosmarcacao()) || $intervaloMarcacao->getNrintervalosmarcacao() == '') {
                    $nrIntervaloMarcacao = \Geral\Util\NovoCodigo::geraCodigo($intervaloMarcacao->getNrorg(), 'PTO_INTERVALOSMARCACAO', 1, 10, true, false);
                    $intervaloMarcacao->setNrintervalosmarcacao($nrIntervaloMarcacao);
                    $intervaloMarcacao->setDtinclusao($dtAtual);
                    $intervaloMarcacao->setNrorginclusao($nrOrg);
                    $intervaloMarcacao->setCdoperinclusao($operador);
                }
                
                // if (empty($intervaloMarcacao->getNrmarcacaopontoini())) {
                //     $intervaloMarcacao->setNrmarcacaopontoini(0);                
                // }
                
                // if (empty($intervaloMarcacao->getNrmarcacaopontofim())) {
                //     $intervaloMarcacao->setNrmarcacaopontofim(0);
                // }

                $intervaloMarcacao->setDtultatu($dtAtual);
                $intervaloMarcacao->setNrorgultatu($nrOrg);
                $intervaloMarcacao->setCdoperultatu($operador);
                
                $intervaloMarcacao->persist();
            }
            \Setforms2\Shared\Registry::getEntityManager()->flush(); //Sincroniza com o banco
            \Setforms2\Shared\Registry::getEntityManager()->clear(); //Limpa os dados do EntityManager
        } catch (\Exception $e) {
            //\Setforms2\Shared\Registry::createManager();
            throw new \Exception('Problema na inserção do intervalo de '.$intervaloMarcacao->getMarcacaopontoini()->getDthrajustada().' a '.$intervaloMarcacao->getMarcacaopontofim()->getDthrajustada());
        }

        return $intervalosMarcacao;
    }

    /**
     * @param $dia Dia
     * @return Dia
     */
    private function organizaIntervalosComNumerosMarcacao($dia){
        $registrosNDesprezados = $dia->getRegistrosNDesprez();
        $intervalosAlterRegistros = array();
        foreach ($registrosNDesprezados as $key => $registro) {
            if ($key == 0 && count($registrosNDesprezados) > 1) {
                $intervalosAlterRegistros[]   = array('MARCINI' => $registro,
                    'MARCFIM'     => isset($registrosNDesprezados[$key + 1]) ? $registrosNDesprezados[$key + 1] : null);
            } elseif ($key == (count($registrosNDesprezados) - 2)) {
                $intervalosAlterRegistros[]   = array('MARCINI' => $registro,
                    'MARCFIM'     => isset($registrosNDesprezados[$key + 1]) ? $registrosNDesprezados[$key + 1] : null);
            } elseif ($key != (count($registrosNDesprezados) - 1)) {
                $intervalosAlterRegistros[]   = array('MARCINI' => $registro,
                    'MARCFIM'     => isset($registrosNDesprezados[$key + 1]) ? $registrosNDesprezados[$key + 1] : null);
            }
        }
        $intervalosRegistros = $dia->getIntervalosRegistros();
        /** @var PtoIntervalosmarcacao $intervalosAnterioresRegistros */
        foreach($intervalosRegistros as $key => &$intervaloRegistro){
            $intervaloRegistro->setMarcacaopontoini($intervalosAlterRegistros[$key]['MARCINI']->getMarcacao());
            $intervaloRegistro->setMarcacaopontofim($intervalosAlterRegistros[$key]['MARCFIM']->getMarcacao());
        }

        $dia->setIntervalosRegistros($intervalosRegistros);
        return $dia;
    }

    public function retornaNomeVinculo(){
        $vinculo = GpeVinculom::findOneByNrvinculomAndNrorg($this->nrvinculom, $this->nrorg);
        $nmVinculom = '';
        if(is_object($vinculo)){
            $nmVinculom =$vinculo->getNmvinculom();
        }
        return $nmVinculom;
    }

    /**
     * Retorna o historico de escala de uma determinada escala
     * @param string $nrEscalaTrabm
     * @return GpeEscalatrabh | null
     */
    public function getHistoricoEscala(string $nrEscalaTrabm): ?GpeEscalatrabh {
        $whereEscalaTrabH = "NRESCALATRABM = ?
                        AND NRORG = ?
                        AND DTMESCOMPETENC IN (SELECT MAX(ETH.DTMESCOMPETENC)
                                                 FROM GPE_ESCALATRABH ETH
                                                WHERE ETH.NRESCALATRABM  = ?
                                                  AND ETH.NRORG = ?
                                                  AND ETH.DTMESCOMPETENC <= TO_DATE(?, 'DD/MM/YYYY'))";

        $paramsEscalaTrabH = array(
            $nrEscalaTrabm,
            $this->nrorg,
            $nrEscalaTrabm,
            $this->nrorg,
            FPA::getDtCompetencia(),
        );

        return GpeEscalatrabh::getObjectByWhere($whereEscalaTrabH, $paramsEscalaTrabH);
    }
}