<?php

namespace HCM\FPA\Entity;

use HCM\Util\Repositories;
use HCM\Util\MsgCalculo;
use HCM\Util\DateUtil;
use HCM\Util\TkEntity;
use Zeedhi\Framework\ORM\DateTime;
use HCM\FPA\Calculos\Geral\FpatpcalcmovLogic;
use HCM\FPA\Entity\FpaitecalcfolhaEntity;


class FpatpcalcmovEntity extends TkEntity
{
    private $_nrs_calculo = array();
    private $competencia;

    const TPCALCMOV_IDPERSISTE_SIM='S';
    const TPCALCMOV_IDPERSISTE_NAO='N';
    
    public function __construct($entityManager, $hcmParameters) {
        $this->hcmParameters = $hcmParameters;
        
        $this->iteCalcFolha = Repositories::FPA_ITECALCFOLHA;
        parent::__construct(null,$entityManager);
    }

    public function getNrcalculo() {
        return $this->_nrs_calculo;
    }
    
    public function getCompetencia() {
        return $this->competencia;
    }
    
    public function setCompetencia($competencia) {
        $this->competencia = $competencia;
    }

    public function setNrcalculo($nr_calculo, $nr_tpcalcmov, $indice) {
        $this->_nrs_calculo[$nr_tpcalcmov][$indice] = $nr_calculo;
        return $this->_nrs_calculo[$nr_tpcalcmov][$indice];
    }

    public function retornaTipoCalculoMovimento($nr_tipocalculo) {

        $resultado = $this->entityManager->getRepository(Repositories::FPA_TPCALCMOV)->findBy(array('nrtipocalculo' => $nr_tipocalculo));
        
        if ($resultado !== null) {
            foreach ($resultado as $entity_obj) {
                $this->addItem($entity_obj, $entity_obj->getNrtpcalcmov());
            }
        } else {
            $message = 'FPATPCALCMOVENTITY  Nao foi encontrado movimentos para tipo de calculo ';
            $message .= $nr_tipocalculo.'<br/>';
            throw new Exception($message);
        }
    }

    private function _adicionaNrCalculo($nr_def, $nr_calc) {
        $tp_calc_mov_logic = new FpatpcalcmovLogic($nr_calc, new FpaitecalcfolhaEntity($this->entityManager, $this->hcmParameters, $nr_calc, $this->competencia));
        $definicoes =& $this->_nrs_calculo[$nr_def];
        $definicoes[] = $tp_calc_mov_logic;
        return $tp_calc_mov_logic;
    }

    public function atribuiNrCalculoFolha(
        $nrorg, $calculo_folha, $dt_ocorrencia, $nrTpModalidadeCalc, $nrOcorrenciaCalc
    ){
        $itens = $this->getItems();
        $dt_ocorrencia = DateUtil::getDataDeString(substr($dt_ocorrencia, 0, 10), DateUtil::FORMATO_BRASILEIRO, true);  
        
        foreach ($itens as $item) {
            
            if ((int)$item->getQtdeslocamento() !== 0) {
                $dataDeslocada = DateUtil::addSubIntervalo(substr($dt_ocorrencia, 0, 10), (int)$item->getQtdeslocamento(), DateUtil::MES);
            }else{
                $dataDeslocada = $dt_ocorrencia;
            }
            $calc_folha = $calculo_folha->retornaCalculoFolha(
                $nrorg, $item->getNrtipomoviment(), $dataDeslocada, $nrTpModalidadeCalc, $nrOcorrenciaCalc
            );
            
            foreach ($calc_folha as $calc) {
                $this->_adicionaNrCalculo($item->getNrtpcalcmov(), $calc->getNrcalculofolha());
            }
        }
    }

    public function insereNrCalculoFolha(
     $calculo_folha, $nrorg, $mesCompetencia, $nrTipoMovimento, $nrTpModalidadeCalc,
        $nrOcorrenciaCalc
    ){

         $nrCalcFolha = $calculo_folha->inserePersisteCalculoFolha(
             $nrorg, $nrTipoMovimento, $nrTpModalidadeCalc, $nrOcorrenciaCalc, $mesCompetencia
         );
         
         $objTpCalcMov = $this->retornaTpCalcMovPorMovimento($nrTipoMovimento);

         $this->_adicionaNrCalculo($objTpCalcMov->getNrtpcalcmov(), $nrCalcFolha);

        return $this;
    }

    public function retornaItensCalculo($vinculo) {
        foreach ($this->_nrs_calculo as $nr_def => $itens) {
            $definicao = $this->_procuraTpCalcMov($nr_def);
            foreach ($itens as $posicao => $item) {
                $nr_calculo_folha = $item->getNrCalcFolha();
                $instancia = $item->getInstancia();
                $conta_itens = $instancia->retornaItensPorVinculo(
                    $vinculo->getNrorg(), $vinculo->getNrvinculom(), $nr_calculo_folha
                );
                if ((int)$conta_itens === 0) {
                    if ($definicao->getIdprincipal() === 'S') {
                        if ($definicao->getIdpodevazio() === 'N') {
                            return false;
                        }
                    }
                }
            }
        }
        return true;
    }

    public function persisteItensCalculo($nrGozoferias = null, $nrLotevinc = null, $idedmdev = null) {

        foreach ($this->_nrs_calculo as $nr_def => $itens) {
            $definicao = $this->_procuraTpCalcMov($nr_def);
            foreach ($itens as $posicao => $item) {
                if ($definicao->getIdpersiste() === 'S') {
                    $instancia = $item->getInstancia();
                    $items = $instancia->getItems();
                    if (count($items) > 0) {
                        if($nrGozoferias){
                            foreach($items as $item){
                                $item->setNrgozoferias($nrGozoferias);
                            }
                        }
                        
                        if($nrLotevinc){
                           foreach($items as $item){
                                $item->setNrlotevinc($nrLotevinc);
                            } 
                        }
                        
                        if($idedmdev){
                            foreach($items as $item){
                                $item->setIdedmdev($idedmdev);
                            }
                        }
                        $instancia->persist();
                    }
                }
            }
        }
    }
    
    public function apagaItensCalculo(){
        foreach ($this->_nrs_calculo as $nr_def => $itens) {
            $definicao = $this->_procuraTpCalcMov($nr_def);
            if ($definicao->getIdpersiste() == self::TPCALCMOV_IDPERSISTE_SIM) {
                foreach ($itens as $posicao => $item) {
                    $instancia = $item->getInstancia();
                    $apagou = $instancia->apagaItemCalculo();
                    if ($apagou === true) {
                        $instancia->persist();
                    }
                }
            }
        }
    }

    public function retornaListaItensCalculo(
        $mes_ocorrencia, $evento,
        $vinculo
    ) {
        foreach ($this->_nrs_calculo as $nr_def => $itens) {
            foreach ($itens as $posicao => $item) {
                $instancia = $item->getInstancia();
                $definicao = $this->_procuraTpCalcMov($nr_def);
                if ((int)$definicao->getNrordimpreseven() !== 0) {
                    $itens = $instancia->retornaArrayItemCalculo($mes_ocorrencia, $evento, $vinculo, $item->getNrCalcFolha(), $definicao->getNmtpcalcmov());
                    if (count($itens) > 0) {
                        $lista[] = $itens;
                    }
                } elseif ($definicao->getNrordimpreseven() === null) {
                    MsgCalculo::getInstance()->adicionaMensagem('Não foi definida a ordem de impressão no movimento.',MsgCalculo::ALERTA);
                }
            }
        }
        return $lista;
    }

    public function retornaPosicaoTpMov($nr_tp_mov, $deslocamento = 0){
            
        foreach ($this->_nrs_calculo as $nr_def => $item) {
            $definicao = $this->_procuraTpCalcMov($nr_def);
            if (((int)$definicao->getNrtipomoviment() === (int)$nr_tp_mov)
                &&((int)$definicao->getQtdeslocamento() === (int)$deslocamento)
            ) {
                return $item;
            }
        }
        MsgCalculo::getInstance()->adicionaMensagem('Não foi encontrada associção do Tipo de Calculo com o Movimento '.$nr_tp_mov,MsgCalculo::ERRO,true);
    }

    public function retornaTpCalcMovPorMovimento($nrTipoMovimento){
        $itens = $this->getItems();
        foreach ($itens as $item) {
            if ((int)$item->getNrtipomoviment() === (int)$nrTipoMovimento) {
                return $item;
            }
        }
        return false;
    }

    private function _procuraTpCalcMov($nr_def) {
        return $this->getItem($nr_def);
    }

    public function retornaTpMovPrincipal() {
        foreach ($this->getItems() as $item) {
            if ($item->getIdprincipal() === 'S') {
                return $item->getNrtipomoviment();
            }
        }
        $mensagem = 'FPATPCALCMOVENTITY  Nao foi encontrado tipo de movimento principal<br/>';
        throw new Exception($mensagem);
    }
}

