<?php

namespace HCM\FPA\ObrigacaoLegal\Manad;

use HCM\FPA\ObrigacaoLegal\Manad\Blocos\Registros\Registro0000;
use HCM\FPA\ObrigacaoLegal\Manad\Blocos\BlocoK;
use HCM\FPA\ObrigacaoLegal\Manad\QueryManad;

class Manad {

    private $hcmParameters;
    private $_objMagnetico;
    private $_nrEstrResponsa;
    private $_dtUltDiaMes;
    private $_primeiroDiaAno;
    private $_dtPeriodIni;
    private $_dtPeriodFim;
    private $_nrOrg;
    private $_nrOrgPadrao;
    private $_dtCompetencia;
    private $_bloco0;
    private $_blocoK;
    private $_vinculo;
    private $_idCentrEscrit;
    private $_idFinaldArq;
    private $queryManad;
    private $estrutUtilizadas = [];

    public function __construct($entityManager, $hcmParameters, $objMagnetico, $bloco0, $nrEstrResponsa, $dtUltDiaMes, $primeiroDiaAno, $dtPeriodIni, $dtPeriodFim, $idCentrEscrit, $idFinaldArq) {
        $this->entityManager   = $entityManager;
        $this->hcmParameters   = $hcmParameters;
        $this->_objMagnetico   = $objMagnetico;
        $this->_nrEstrResponsa = $nrEstrResponsa;
        $this->_dtUltDiaMes    = $dtUltDiaMes;
        $this->_primeiroDiaAno = $primeiroDiaAno;
        $this->_dtPeriodIni    = $dtPeriodIni;
        $this->_dtPeriodFim    = $dtPeriodFim;
        $this->_nrOrg          = $hcmParameters->getNrorgTrabalho();
        $this->_nrOrgPadrao    = $hcmParameters->getNrorgPadrao();
        $this->_dtCompetencia  = $hcmParameters->getDtCompetencia();
        $this->_bloco0         = $bloco0;
        $this->_idCentrEscrit  = $idCentrEscrit;
        $this->_idFinaldArq    = $idFinaldArq;
        $this->_blocoK         = new BlocoK();
        $this->queryManad      = new QueryManad($this->entityManager,$this->hcmParameters);
    }

    public function geraManad($nrpessoa) {
        $this->_vinculo = $this->getDataPessoa($nrpessoa);

        $this->_preparaRegistro0000();
        $this->_preparaBlocoK();

        $this->estrutUtilizadas[] = $this->_vinculo['CDCNPJEST'];
    }

    private function getDataPessoa($nrpessoa) { 
        try{
            $params = ['P_NRORG'          => $this->_nrOrg,
                       'P_DTMESCOMPETENC' => $this->_dtCompetencia,
                       'P_PRIMEIRODIAANO' => $this->_primeiroDiaAno,
                       'P_DTULTDIAMES'    => $this->_dtUltDiaMes,
                       'P_DTPERIODINI'    => $this->_dtPeriodIni->format('d/m/Y'),
                       'P_DTPERIODFIM'    => $this->_dtPeriodFim->format('d/m/Y'),
                       'P_NRPESSOA'       => $nrpessoa];

            return $this->queryManad->get_dataPessoa($params);
        } catch(\Exception $e){
            throw new \Exception('Erro ao buscar dados da nrpessoa: '.$nrpessoa.'. '.PercorreManad::ql_msg.'Error:'.$e->getMessage());
        }
    }

    private function _preparaRegistro0000(){
        $cpf = null; $cnpj = null;

        try{
            $params = ['P_NRORG'          => $this->_nrOrg,
                       'P_ESTRUTURA'      => $this->_vinculo['NRESTRUTURAM'],
                       'P_DTMESCOMPETENC' => $this->_dtCompetencia];
            
            $sqlResult = $this->queryManad->get_reg0000($params);
        } catch(\Exception $e){
            throw new \Exception('Erro na query: Bloco 0, Registro tipo 0000, item 4.2.'.self::ql_msg.'Error:'.$e->getMessage());
        }

        if(!empty($sqlResult) && !in_array($this->_vinculo['CDCNPJEST'], $this->estrutUtilizadas)) {
            if(!empty($sqlResult['CDCNPJ'])) {
                if($sqlResult['TPINSC'] == 'CNPJ')
                    $cnpj = $sqlResult['CDCNPJ'];
                else
                    $cpf = substr($sqlResult['CDCNPJ'], 0, 11);
            }
            
            $nome = $sqlResult['NOMEEMPRESARIAL'];
            $cei  = $sqlResult['CDCEI'];
            $dtIni = $this->_dtPeriodIni->format('dmY');
            $dtFim = $this->_dtPeriodFim->format('dmY');
            $uf = $sqlResult['CDUF'];
            $codMun = $sqlResult['CDMUN'];
                
            $this->_bloco0->_registro0000->setNome($nome);
            $this->_bloco0->_registro0000->setCnpj($cnpj);
            $this->_bloco0->_registro0000->setCpf($cpf);
            $this->_bloco0->_registro0000->setCei($cei);
            $this->_bloco0->_registro0000->setNit(null);
            $this->_bloco0->_registro0000->setUf($uf);
            $this->_bloco0->_registro0000->setIe(null);
            $this->_bloco0->_registro0000->setCodMun($codMun);
            $this->_bloco0->_registro0000->setIm(null);
            $this->_bloco0->_registro0000->setSuframa(null);
            $this->_bloco0->_registro0000->setIndCentr($this->_idCentrEscrit);
            $this->_bloco0->_registro0000->setDtIni($dtIni);
            $this->_bloco0->_registro0000->setDtFim($dtFim);
            $this->_bloco0->_registro0000->setCodFin($this->_idFinaldArq);

            $this->_bloco0->_registro0000->geraRegistro0000();
            
            $registro = $this->_bloco0->_registro0000->getLinhaRegistro();
            $this->_addRegVinculo(Registro0000::REGISTRO_0000, $registro);

            $this->_bloco0->_registro0000->setLinhaRegistro();
    
            $this->_objMagnetico->fimAddRegVinculo();
        }
    }

    private function _preparaBlocoK(){
        $this->_preparaRegistroK050();
        $this->_preparaRegistroK100();
        $this->_preparaRegistroK250();
        $this->_preparaRegistroK300();
    }

    public function _geraERetornaRegistroK001(){ //Abertura Bloco K
        $this->_blocoK->_registroK001->geraRegistroK001();

        return $this->_blocoK->_registroK001->getLinhaRegistro().Registro0000::QUEBRA_LINHA_CR_LF;
    }

    private function _preparaRegistroK050(){
        try{
            $params = ['P_NRORG'          => $this->_nrOrg,
                       'P_DTMESCOMPETENC' => $this->_dtCompetencia,
                       'P_PRIMEIRODIAANO' => $this->_primeiroDiaAno,
                       'P_DTULTDIAMES'    => $this->_dtUltDiaMes,
                       'P_DTPERIODINI'    => $this->_dtPeriodIni->format('d/m/Y'),
                       'P_DTPERIODFIM'    => $this->_dtPeriodFim->format('d/m/Y'),
                       'P_NRVINCULOM'     => $this->_vinculo['NRVINCULOM']];

            $sqlResult = $this->queryManad->get_regK050($params);
        } catch(\Exception $e){
            throw new \Exception('Erro na query: Bloco K, Registro tipo K050 , item 4.4.'.PercorreManad::ql_msg.'Error:'.$e->getMessage());
        }

        if(!empty($sqlResult)) {
            $cnpjcei = !empty($sqlResult['CDCNPJEST']) ? $sqlResult['CDCNPJEST'] : $sqlResult['CDCEIEST'];
            $dtIncAlt = $sqlResult['DTADMISSAOVINC'];
            $regTrab = $sqlResult['NRVINCULOM'];
            $cpf  = $sqlResult['NRCPFPESSOA'];
            $nit  = $sqlResult['NRPISPASEPPES'];
            $categ = $sqlResult['CODCATEGTRAB'];
            $nome   = $sqlResult['NMVINCULOM'];
            $dtNasc = $sqlResult['DTNASCPESSOA'];
            $dtAdm  = $sqlResult['DTADMISSAOVINC'];
            $dtDem  = $sqlResult['DTRESCISAOVINC'];
            $indVinc = $this->deParaIndTpVinculo($sqlResult['NRVINCULOEMPREG']);
                
            $this->_blocoK->_registroK050->setCnpjCei($cnpjcei);
            $this->_blocoK->_registroK050->setDtIncAlt($dtIncAlt);
            $this->_blocoK->_registroK050->setRegTrab($regTrab);
            $this->_blocoK->_registroK050->setCpf($cpf);
            $this->_blocoK->_registroK050->setNit($nit);
            $this->_blocoK->_registroK050->setCateg($categ);
            $this->_blocoK->_registroK050->setNome($nome);
            $this->_blocoK->_registroK050->setDtNasc($dtNasc);
            $this->_blocoK->_registroK050->setDtAdm($dtAdm);
            $this->_blocoK->_registroK050->setDtDem($dtDem);
            $this->_blocoK->_registroK050->setIndVinc($indVinc);
            $this->_blocoK->_registroK050->setTpAtoNom(null);
            $this->_blocoK->_registroK050->setNrAtoNom(null);
            $this->_blocoK->_registroK050->setDtAtoNom(null);

            $this->_blocoK->_registroK050->geraRegistroK050();
            
            $registro = $this->_blocoK->_registroK050->getLinhaRegistro();
            $this->_addRegVinculo(Registro0000::REGISTRO_K050, $registro);
    
            $this->_blocoK->_registroK050->setLinhaRegistro();
            
            $this->_objMagnetico->fimAddRegVinculo();
        }
    }

    private function _preparaRegistroK100(){
        try{
            $params = ['P_NRORG'              => $this->_nrOrg,
                       'P_DTMESCOMPETENC'     => $this->_dtCompetencia,
                       'P_PRIMEIRODIAANO'     => $this->_primeiroDiaAno,
                       'P_DTULTDIAMES'        => $this->_dtUltDiaMes,
                       'P_DTPERIODINI'        => $this->_dtPeriodIni->format('d/m/Y'),
                       'P_DTPERIODFIM'        => $this->_dtPeriodFim->format('d/m/Y'),
                       'P_NRVINCULOM'         => $this->_vinculo['NRVINCULOM'],
                       'P_NRTIPOESTRUTGEREN'  => $this->hcmParameters->getTipoEstruturaGerencialOrg(),
                       'P_NRTIPOESTRUTOMADOR' => $this->hcmParameters->getTipoEstruturaTomadorOrg()];

            $sqlResult = $this->queryManad->get_regK100($params);
        } catch(\Exception $e){
            throw new \Exception('Erro na query: Bloco K, Registro tipo K100 , item 4.4.'.PercorreManad::ql_msg.'Error:'.$e->getMessage());
        }

        if(!empty($sqlResult) && !in_array($this->_vinculo['CDCNPJEST'], $this->estrutUtilizadas)) {
            $dtIncAlt = $sqlResult['DTINIVIGENCIA'];
            $codLtc  = $sqlResult['NRESTRUTURAM'];
            $cnpjcei = !empty($sqlResult['CDCNPJEST']) ? $sqlResult['CDCNPJEST'] : $sqlResult['CDCEIEST'];
            $descLtc = $sqlResult['NMRAZSOCESTRUT'];
            $cnpjCeiTom  = !empty($sqlResult['CDCNPJTOM']) ? $sqlResult['CDCNPJTOM'] : $sqlResult['CDCEITOM'];
                
            $this->_blocoK->_registroK100->setDtIncAlt($dtIncAlt);
            $this->_blocoK->_registroK100->setCodLtc($codLtc);
            $this->_blocoK->_registroK100->setCnpjCei($cnpjcei);
            $this->_blocoK->_registroK100->setDescLtc($descLtc);
            $this->_blocoK->_registroK100->setCnpjCeiTom($cnpjCeiTom);

            $this->_blocoK->_registroK100->geraRegistroK100();
            
            $registro = $this->_blocoK->_registroK100->getLinhaRegistro();
            $this->_addRegVinculo(Registro0000::REGISTRO_K100, $registro);
    
            $this->_blocoK->_registroK100->setLinhaRegistro();

            $this->_objMagnetico->fimAddRegVinculo();
        }
    }

    public function _preparaRegistroK150(){ //registro geral fora de vínculo.
        $cnpjcei = null;
        
        foreach($this->_vinculo as &$value) {
            $value = null;
        }

        $estrutResp = $this->getDataEstrutResp();

        try{
            $params = ['P_NRORG'          => $this->_nrOrg,
                       'P_DTMESCOMPETENC' => $this->_dtCompetencia,
                       'P_DTPERIODINI'    => $this->_dtPeriodIni->format('d/m/Y'),
                       'P_DTPERIODFIM'    => $this->_dtPeriodFim->format('d/m/Y')];

            $sqlResult = $this->queryManad->get_regK150($params);
        } catch(\Exception $e){
            throw new \Exception('Erro na query: Bloco K, Registro tipo K150, item 4.4.'.PercorreManad::ql_msg.'Error:'.$e->getMessage());
        }
        
        foreach($sqlResult as $regK150) {
            if(!empty($estrutResp))
                $cnpjcei = !empty($estrutResp['CDCNPJ']) ? $estrutResp['CDCNPJ'] : $estrutResp['CDCEI'];

            $dtIncAlt = $regK150['DTMESCOMPETENC'];
            $cobRub   = $regK150['NREVENTOM'];
            $descRub  = $regK150['NMEVENTOH'];
                
            $this->_blocoK->_registroK150->setCnpjCei($cnpjcei);
            $this->_blocoK->_registroK150->setDtIncAlt($dtIncAlt);
            $this->_blocoK->_registroK150->setCobRub($cobRub);
            $this->_blocoK->_registroK150->setDescRub($descRub);

            $this->_blocoK->_registroK150->geraRegistroK150();
            
            $registro = $this->_blocoK->_registroK150->getLinhaRegistro();
            $this->_addRegVinculo(Registro0000::REGISTRO_K150, $registro);

            $this->_blocoK->_registroK150->setLinhaRegistro();
    
            $this->_objMagnetico->fimAddRegVinculo();
        }
    }

    private function _preparaRegistroK250(){
        try{
            $params = ['P_NRORG'          => $this->_nrOrg,
                       'P_DTMESCOMPETENC' => $this->_dtCompetencia,
                       'P_NRVINCULOM'     => $this->_vinculo['NRVINCULOM'],
                       'P_MODALIDADECAL'  => $this->hcmParameters->getNrModalidadeCalculo(),
                       'P_PRIMEIRODIAANO' => $this->_primeiroDiaAno,
                       'P_DTULTDIAMES'    => $this->_dtUltDiaMes,
                       'P_DTPERIODINI'    => $this->_dtPeriodIni->format('d/m/Y'),
                       'P_DTPERIODFIM'    => $this->_dtPeriodFim->format('d/m/Y')];

            $sqlResult = $this->queryManad->get_regK250($params);
        } catch(\Exception $e){
            throw new \Exception('Erro na query: Bloco K, Registro tipo K250 , item 4.4.'.PercorreManad::ql_msg.'Error:'.$e->getMessage());
        }
        
        foreach($sqlResult as $regK250) {
            $cnpjcei = !empty($regK250['CDCNPJEST']) ? $regK250['CDCNPJEST'] : $regK250['CDCEIEST'];
            $indFl   = $this->deParaIndTpFolha($regK250['NRTIPOMOVIMENT']);
            $codLtc  = $regK250['NRESTRUTURAM'];
            $regTrab = $regK250['NRVINCULOM'];
            $dtComp = $regK250['DT_COMP'];
            $dtPgto = $regK250['DT_PGTO'];
            $cob    = $regK250['NRCBO'];
            $ocorr  = $this->deParaCodOcorrencia($this->_vinculo['NRAGENTENOCIVO'], $this->_vinculo['IDMULTVINC']);
            $descCargo  = $regK250['NMCARGO'];
            $qtdDepIr   = $regK250['QTD_DEP_IR'];
            $qtdDepSf   = $regK250['QTD_DEP_SF'];
            $vlBaseIrrf = $regK250['BASE_IRRF_VALOR'];
            $vlBasePs   = $regK250['VL_BASE_PS'];
            $nrcalculofolha = $regK250['NRCALCULOFOLHA'];

            $this->_blocoK->_registroK250->setCnpjCei($cnpjcei);
            $this->_blocoK->_registroK250->setIndFl($indFl);
            $this->_blocoK->_registroK250->setCodLtc($codLtc);
            $this->_blocoK->_registroK250->setRegTrab($regTrab);
            $this->_blocoK->_registroK250->setDtComp($dtComp);
            $this->_blocoK->_registroK250->setDtPgto($dtPgto);
            $this->_blocoK->_registroK250->setCob($cob);
            $this->_blocoK->_registroK250->setOcorr($ocorr);
            $this->_blocoK->_registroK250->setDescCargo($descCargo);
            $this->_blocoK->_registroK250->setQtdDepIr($qtdDepIr);
            $this->_blocoK->_registroK250->setQtdDepSf($qtdDepSf);
            $this->_blocoK->_registroK250->setVlBaseIrrf($vlBaseIrrf);
            $this->_blocoK->_registroK250->setVlBasePs($vlBasePs);

            $this->_blocoK->_registroK250->geraRegistroK250();
            
            $registro = $this->_blocoK->_registroK250->getLinhaRegistro();
            $this->_addRegVinculo(Registro0000::REGISTRO_K250, $registro, $vlBaseIrrf, $vlBasePs, NULL, $nrcalculofolha);

            $this->_blocoK->_registroK250->setLinhaRegistro();
    
            $this->_objMagnetico->fimAddRegVinculo();
        }
    }

    private function _preparaRegistroK300(){
        try{
            $params = ['P_NRORG'          => $this->_nrOrg,
                       'P_NRORGPADRAO'    => $this->_nrOrgPadrao,
                       'P_DTMESCOMPETENC' => $this->_dtCompetencia,
                       'P_NRVINCULOM'     => $this->_vinculo['NRVINCULOM'],
                       'P_MODALIDADECAL'  => $this->hcmParameters->getNrModalidadeCalculo(),
                       'P_PRIMEIRODIAANO' => $this->_primeiroDiaAno,
                       'P_DTULTDIAMES'    => $this->_dtUltDiaMes,
                       'P_DTPERIODINI'    => $this->_dtPeriodIni->format('d/m/Y'),
                       'P_DTPERIODFIM'    => $this->_dtPeriodFim->format('d/m/Y')];

            $sqlResult = $this->queryManad->get_regK300($params);
        } catch(\Exception $e){
            throw new \Exception('Erro na query: Bloco K, Registro tipo K300 , item 4.4.'.PercorreManad::ql_msg.'Error:'.$e->getMessage());
        }
        
        foreach($sqlResult as $regK300) {
            $cnpjcei = !empty($regK300['CDCNPJEST']) ? $regK300['CDCNPJEST'] : $regK300['CDCEIEST'];
            $indFl   = $this->deParaIndTpFolha($regK300['NRTIPOMOVIMENT']);
            $codLtc  = $regK300['NRESTRUTURAM'];
            $regTrab = $regK300['NRVINCULOM'];
            $dtComp = $regK300['DT_COMP'];
            $cobRub = $regK300['NREVENTOM'];
            $vlrRubr = $regK300['VREVENTOFOLHA'];
            $indRubr = $regK300['IND_RUBR'];
            $indBaseIrrf = $regK300['IND_BASE_IRRF'];
            $indBasePs   = $regK300['IND_BASE_PS'];
            $nrcalculofolha = $regK300['NRCALCULOFOLHA'];

            $this->_blocoK->_registroK300->setCnpjCei($cnpjcei);
            $this->_blocoK->_registroK300->setIndFl($indFl);
            $this->_blocoK->_registroK300->setCodLtc($codLtc);
            $this->_blocoK->_registroK300->setRegTrab($regTrab);
            $this->_blocoK->_registroK300->setDtComp($dtComp);
            $this->_blocoK->_registroK300->setCobRub($cobRub);
            $this->_blocoK->_registroK300->setVlrRubr($vlrRubr);
            $this->_blocoK->_registroK300->setIndRubr($indRubr);
            $this->_blocoK->_registroK300->setIndBasePs($indBasePs);
            $this->_blocoK->_registroK300->setIndBaseIrrf($indBaseIrrf);

            $this->_blocoK->_registroK300->geraRegistroK300();
            
            $registro = $this->_blocoK->_registroK300->getLinhaRegistro();
            $this->_addRegVinculo(Registro0000::REGISTRO_K300, $registro, $indBaseIrrf, $indBasePs, $vlrRubr, $nrcalculofolha);

            $this->_blocoK->_registroK300->setLinhaRegistro();
    
            $this->_objMagnetico->fimAddRegVinculo();
        }
    }

    public function _geraERetornaRegistroK990($countRegK){ //Finalização Bloco K
        $this->_blocoK->_registroK990->setQtdLinhas($countRegK);
        $this->_blocoK->_registroK990->geraRegistroK990();

        return $this->_blocoK->_registroK990->getLinhaRegistro().Registro0000::QUEBRA_LINHA_CR_LF;
    }

    private function deParaIndTpVinculo($nrvinculoempreg){
        $value = null;

        switch($nrvinculoempreg){
            case '1':  //Trabalhador urbano vinculado a empregador pessoa jurídica por contrato de trabalho regido pela CLT, por prazo indeterminado
            case '2':  //Trabalhador urbano vinculado a empregador pessoa física por contrato de trabalho regido pela CLT, por prazo indeterminado
            case '3':  //Trabalhador rural vinculado a empregador pessoa jurídica por contrato de trabalho regido pela Lei nº 5.889/1973, por prazo indeterminado
            case '4':  //Trabalhador rural vinculado a empregador pessoa física por contrato de trabalho regido pela Lei nº 5.889/1973, por prazo indeterminado
            case '10': //Trabalhador urbano vinculado a empregador pessoa jurídica por contrato de trabalho regido pela CLT, por tempo determinado ou obra certa
            case '11': //Trabalhador urbano vinculado a empregador pessoa física por contrato de trabalho regido pela CLT, por tempo determinado ou obra certa
            case '12': //Trabalhador rural vinculado a empregador pessoa jurídica por contrato de trabalho regido pela Lei nº 5.889/1973, por prazo determinado
            case '13': //Trabalhador rural vinculado a empregador pessoa física por contrato de trabalho regido pela Lei nº 5.889/1973, por prazo determinado
                $value = 1; //vínculo normal
                break;
            case '8':  //Trabalhador temporário, regido pela Lei no 6.019, de 3 de janeiro de 1974
                $value = 2; //vínculo temporário com redução de contribuições/obrigação
                break;
            case '5':  //Servidor regido pelo Regime Jurídico Único (federal, estadual e municipal) e militar, vinculado a Regime Próprio de Previdência
                $value = 3; //Servidor público Efetivo Estatutário
                break;
            case '22': //Servidor regido pelo Regime Jurídico Único (federal, estadual e municipal) e militar, vinculado ao Regime Geral de Previdência Social
                $value = 4; //Servidor público Efetivo CLT
                break;
            case '6':  //Servidor público não efetivo (demissível ad nutum ou admitido por meio de legislação especial, não-regido pela CLT)
                $value = 5; //Servidor público Comissionado
                break;
            case '14': //Contribuinte individual - Diretor não empregado e demais empresários sem FGTS
            case '21': //Contribuinte individual – Trabalhador autônomo ou a este equiparado, inclusive o operador de máquina, com contribuição sobre remuneração; trabalhador associado à cooperativa de produção
            case '24': //Contribuinte Individual - Transportador autônomo, com contribuição sobre remuneração.
            case '26': //Contribuinte individual - Diretor não empregado com FGTS (Lei nº 8.036/90, art.16);
            case '27': //Contribuinte individual – Microempreendedor individual
            case '31': //Contribuinte individual - Empresário, sócio e membro de conselho de administração ou fiscal
                $value = 8; //Contribuinte Individual
                break;
            default:
                $value = 9; //Outros vínculos
                break;
        }
        return $value;
    }

    private function deParaIndTpFolha($nrtipomoviment){
        $value = null;

        switch($nrtipomoviment){
            case '1':  //MENSAL
                if(is_null($this->_vinculo['DTRESCISAOVINC']))
                    $value = 1; //folha normal
                else
                    $value = 6; //outras folhas
                break;
            case '2': //FERIAS
                $value = 3; //folha de férias
                break;
            case '4':  //13 SALARIO
                $value = 2; //folha de 13º salário
                break;
            default:
                $value = 6; //outras folhas
                break;
        }
        return $value;
    }

    private function deParaCodOcorrencia($nragentenocivo, $idmultvinc) {

        $agentenocivo = $this->entityManager->getRepository('Teknisa\Model\Gestaopessoas\Entities\GpeAgentenocivo')->findOneBy(['nragentenocivo' => $nragentenocivo]);

        if (is_object($agentenocivo) && $agentenocivo->getCdagentenocivo()) {
            return $agentenocivo->getCdagentenocivo();
        } else {
            if ($idmultvinc !== Registro0000::IDMULTVINC_UNICO) {
                return '05';
            } else {
                return '00';
            }
        }
    }

    private function getDataEstrutResp() {
        try{
            $params = ['P_NRORG'          => $this->_nrOrg,
                       'P_ESTRUTURA'      => $this->_nrEstrResponsa,
                       'P_DTMESCOMPETENC' => $this->_dtCompetencia];
            
           return $this->queryManad->get_reg0000($params);
        } catch(\Exception $e){
            throw new \Exception('Erro na query: Bloco K, Registro tipo K150, item 4.4.'.self::ql_msg.'Error:'.$e->getMessage());
        }
    }

    private function _addRegVinculo($tpRegistro, $linhaRegistro, $vrbaseirrf = null, $vrbaseps = null, $vrrubr = null, $nrcalculofolha = null) {
        $this->_objMagnetico->addRegVinculo(
            $this->_vinculo['NRCPFPESSOA'], $this->_vinculo['NMVINCULOM'], $this->_vinculo['DTADMISSAOVINC'], $tpRegistro, $linhaRegistro, $this->_vinculo['NRESTRUTLEGAL'],
            $this->_vinculo['NRVINCULOM'], $vrbaseirrf, $vrbaseps, $vrrubr, $nrcalculofolha);
    }

}