<?php
namespace HCM\Geral\Controllers;

use HCM\Geral\Services\PreFechamentoFgtsService;
use HCM\Util\HCMParameters;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DTO\Request;
use Zeedhi\Framework\DTO\Response;

class PreFechamentoFgtsController {
    
    public function __construct (HCMParameters $hcmParameters, PreFechamentoFgtsService $preFechamentoFgtsService) {
        $this->hcmParameters = $hcmParameters;
        $this->preFechamentoFgtsService = $preFechamentoFgtsService;
        $this->conferenciaFgts = new ConferenciaFgts($hcmParameters, $preFechamentoFgtsService);
        $this->informacoesSinteticasFgts = new InformacoesSinteticasFgts($hcmParameters, $preFechamentoFgtsService);
        $this->informacoesSobreBasesCalculoFgts = new InformacoesSobreBasesCalculoFgts($hcmParameters, $preFechamentoFgtsService);
    }
    
    public function getConferenciaFgts(DTO\Request\DataSet $request, DTO\Response $response) {
        $this->conferenciaFgts->getConferenciaFgts($request->getDataSet()->getRows()[0], $response);
    }
    
    public function getInformacoesSinteticasFgts(DTO\Request\DataSet $request, DTO\Response $response) {
        $this->informacoesSinteticasFgts->getInformacoesSinteticasFgts($request->getDataSet()->getRows()[0], $response);
    }
    
    public function getInformacoesSobreBasesCalculoFgts(DTO\Request\DataSet $request, DTO\Response $response) {
        $this->informacoesSobreBasesCalculoFgts->getInformacoesSobreBasesCalculoFgts($request->getDataSet()->getRows()[0], $response);
    }

}

class ConferenciaFgts {
    
    public function __construct (HCMParameters $hcmParameters, PreFechamentoFgtsService $preFechamentoFgtsService) {
        $this->hcmParameters = $hcmParameters;
        $this->preFechamentoFgtsService = $preFechamentoFgtsService;
    }
    
    public function getConferenciaFgts($row, DTO\Response $response) {
        try {
            $parameters = array( 
                'NRORG'          => $row['NRORG'],
                'DTMESCOMPETENC' => $row['DATACOMPETENCIAINICIAL'],
                'NRESTRUTLEGAL'  => isset($row['NRESTRUTLEGAL_LIST']) ? $row['NRESTRUTLEGAL_LIST'] : '',
                'NRTABELA'       => isset($row['APURACAO']) ? $row['APURACAO'] : null
            );

            $conferenciaFgts = $this->preFechamentoFgtsService->getConferenciaFgts($parameters);
            
            $response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet('dataset', $conferenciaFgts));
        } catch(\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }
}

class InformacoesSinteticasFgts {
    
    public function __construct (HCMParameters $hcmParameters, PreFechamentoFgtsService $preFechamentoFgtsService) {
        $this->hcmParameters = $hcmParameters;
        $this->preFechamentoFgtsService = $preFechamentoFgtsService;
    }
    
    public function getInformacoesSinteticasFgts($row, DTO\Response $response) {
        try {
            $parameters = array( 
                'NRORG'          => $row['NRORG'],
                'DTMESCOMPETENC' => $row['DATACOMPETENCIAINICIAL'],
                'NRESTRUTLEGAL'  => isset($row['NRESTRUTLEGAL_LIST']) ? $row['NRESTRUTLEGAL_LIST'] : '',
                'NRTABELA'       => isset($row['APURACAO']) ? $row['APURACAO'] : null
            );

            $informacoesSinteticasFgts = $this->preFechamentoFgtsService->getInformacoesSinteticasFgts($parameters);
            
            $response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet('dataset', $informacoesSinteticasFgts));
        } catch(\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }
}

class InformacoesSobreBasesCalculoFgts {
    
    public function __construct (HCMParameters $hcmParameters, PreFechamentoFgtsService $preFechamentoFgtsService) {
        $this->hcmParameters = $hcmParameters;
        $this->preFechamentoFgtsService = $preFechamentoFgtsService;
    }
    
    public function getInformacoesSobreBasesCalculoFgts($row, DTO\Response $response) {
        try {
            $parameters = array( 
                'NRORG'          => $row['NRORG'],
                'DTMESCOMPETENC' => $row['DATACOMPETENCIAINICIAL'],
                'NRESTRUTLEGAL'  => isset($row['NRESTRUTLEGAL_LIST']) ? $row['NRESTRUTLEGAL_LIST'] : '',
                'NRTABELA'       => isset($row['APURACAO']) ? $row['APURACAO'] : null
            );

            $informacoesSobreBasesCalculoFgts = $this->preFechamentoFgtsService->getInformacoesSobreBasesCalculoFgts($parameters);
            
            $response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet('dataset', $informacoesSobreBasesCalculoFgts));
        } catch(\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }
}