<?php

namespace HCM\Geral\Logic\Esocial;

use HCM\Util\Builder\Template;
use HCM\Util\DateUtil;
use HCM\Util\HCMParameters;
use HCM\Util\Messages;
use HCM\Util\MsgProcesso;
use HCM\Util\Repositories;

class ESocialXmlFactory {
    
    private $_xmlTemplatesPath;
    private $_xsdTemplatesPath;
    
    private $templateDir;
    private $templateTwig;
    
    private $_arrXmlCaracter;
    private $_arrXmlEscape;

    public function __construct($nmEvento, $dsVersao)
    {
        $this->_xmlTemplatesPath = ESOCIAL_FILES_PATH . DIRECTORY_SEPARATOR . "templates" . DIRECTORY_SEPARATOR . "xml";
        $this->_xsdTemplatesPath = ESOCIAL_FILES_PATH . DIRECTORY_SEPARATOR . "templates" . DIRECTORY_SEPARATOR . "xsd";
        
        $this->templateDir = $this->_xmlTemplatesPath . "/v_$dsVersao";
        $this->templateTwig = "{$nmEvento}.xml.twig";
        
        $this->schemaDir = $this->_xsdTemplatesPath . "/v_$dsVersao";
        $this->schemaXsd = "{$nmEvento}-v_{$dsVersao}.xsd";

        $this->_arrXmlCaracter = array('>', '<', '&', '"', "'");
        $this->_arrXmlEscape = array('&gt;', '&lt;', '&amp;', '&quot;', '&#39');
    }

    /**
     * @param $xml
     * @return mixed|\SimpleXMLElement|string
     */
    private function formatXML($xml)
    {
        // Substitui caracteres especias que devem ser evitados ao enviar o xml para o eSocial
        $xml = str_replace($this->_arrXmlEscape, $this->_arrXmlCaracter, $xml);

        // Este bloco transforma o texto gerado, para que as tags do twig sejam ocultadas, a fim de possibilitar a
        // formatacao do documentos, que somente e disponivel para um texto no formato .xml
        $xml = str_replace("{{", "aaa{{", $xml);
        $xml = str_replace("}}", "}}aaa", $xml);
        $xml = str_replace("{%", "<TEMPORARY_TAG>aaa{%", $xml);
        $xml = str_replace("%}", "%}aaa</TEMPORARY_TAG>", $xml);

        $xml = str_replace("&lt;", "<", $xml);
        $xml = str_replace("&gt;", ">", $xml);
        $xml = str_replace("&quot;", '"', $xml);
        $xml = str_replace("&", "e", $xml);     // Retirado carater '&' pois estava dando erro quando existia em algum campo de descrição

        $xml    = new \SimpleXMLElement($xml);
        $domxml = new \DOMDocument();
        $domxml->preserveWhiteSpace = false;
        $domxml->formatOutput = true;
        $domxml->loadXML($xml->asXML());

        // Remocao do texto temporario gerado.
        //$xml =  $domxml->saveHTML();
        $xml =  $domxml->saveXML();
        $xml = str_replace("aaa{{", "{{", $xml);
        $xml = str_replace("}}aaa", "}}", $xml);
        $xml = str_replace("<TEMPORARY_TAG>aaa{%", "{%", $xml);
        $xml = str_replace("%}aaa</TEMPORARY_TAG>", "%}", $xml);

        // Remove a tag XML "<?xml version='1.0'>" pois não é utilizada ao montar o SOAP para envio
        $xml = preg_replace('!^[^>]+>(\r\n|\n)!','',$xml);

        return $xml;
    }

    /**
     * @param \DOMDocument $domXml
     * @param $tag
     * @param $chavePublica
     * @param $chavePrivada
     * @return \DOMDocument
     * @throws \Exception
     */
    private function assinaXmlAux(\DOMDocument $domXml, $tag, $chavePublica, $chavePrivada) {
        try {
            $nodes = $domXml->getElementsByTagName($tag);
            foreach ($nodes as $node) {
                $root = $node->parentNode;
                $id = trim($node->getAttribute("Id"));
                //$dados = $node->C14N(false, false, null, null);
                $dados = $root->C14N(false, false, null, null);
                //$hashValue = hash("sha1", $dados, true);
                $hashValue = hash("sha256", $dados, true);
                $digValue = base64_encode($hashValue);

                $signature = $domXml->createElement("Signature");
                $root->appendChild($signature);
                $signature->setAttribute("xmlns", "http://www.w3.org/2000/09/xmldsig#");
                //$signature->setAttribute("Id", "Ass_" . $id);

                $signedInfo = $domXml->createElement("SignedInfo");
                $signature->appendChild($signedInfo);

                $novaTag = $domXml->createElement("CanonicalizationMethod");
                $signedInfo->appendChild($novaTag);
                $novaTag->setAttribute("Algorithm", "http://www.w3.org/TR/2001/REC-xml-c14n-20010315");

                $novaTag = $domXml->createElement("SignatureMethod");
                $signedInfo->appendChild($novaTag);
                //$novaTag->setAttribute("Algorithm", "http://www.w3.org/2000/09/xmldsig#rsa-sha1");
                $novaTag->setAttribute("Algorithm", "http://www.w3.org/2001/04/xmldsig-more#rsa-sha256");

                $reference = $domXml->createElement("Reference");
                $signedInfo->appendChild($reference);
                //$reference->setAttribute("URI", "#" . $id);
                $reference->setAttribute("URI", "");

                $transforms = $domXml->createElement("Transforms");
                $reference->appendChild($transforms);

                $novaTag = $domXml->createElement("Transform");
                $transforms->appendChild($novaTag);
                $novaTag->setAttribute("Algorithm", "http://www.w3.org/2000/09/xmldsig#enveloped-signature");

                $novaTag = $domXml->createElement("Transform");
                $transforms->appendChild($novaTag);
                $novaTag->setAttribute("Algorithm", "http://www.w3.org/TR/2001/REC-xml-c14n-20010315");

                $novaTag = $domXml->createElement("DigestMethod");
                $reference->appendChild($novaTag);
                //$novaTag->setAttribute("Algorithm", "http://www.w3.org/2000/09/xmldsig#sha1");
                $novaTag->setAttribute("Algorithm", "http://www.w3.org/2001/04/xmlenc#sha256");

                $novaTag = $domXml->createElement("DigestValue", $digValue);
                $reference->appendChild($novaTag);

                $dados = $signedInfo->C14N(false, false, null, null);
                $assinatura = "";
                openssl_sign($dados, $assinatura, $chavePrivada, "SHA256");
                $signatureValue = base64_encode($assinatura);


                $novaTag = $domXml->createElement("SignatureValue", $signatureValue);
                $signature->appendChild($novaTag);

                $keyInfo = $domXml->createElement("KeyInfo");
                $signature->appendChild($keyInfo);

                $X509Data = $domXml->createElement("X509Data");
                $keyInfo->appendChild($X509Data);

                $novaTag = $domXml->createElement("X509Certificate", $chavePublica);
                $X509Data->appendChild($novaTag);
            }
            return $domXml;
        } catch(\Exception $e) {
            throw $e;
        }
    }

    /**
     * @param $data
     * @return string   return $xml string if save is false | $fileName id save is true
     * @throws \Exception
     */
    public function buildXml($data) {
        try {
            $templateBuilder = new Template($data, $this->templateDir);
            
            $xml = $templateBuilder->build($this->templateTwig);
            //$xml = $this->formatXML($xml);
            
            return $xml;
        } catch(\Exception $e) {
            throw $e;
        }
    }

    /**
     * @param $xmlEvento
     * @return string
     * @throws \Exception
     */
    public function assinaXmlEvento($xmlEvento, $chavePublica, $chavePrivada) {
        try {
            /* Formata o xml*/
            $domXml = new \DOMDocument("1.0", "UTF-8");
            $domXml->loadXML($xmlEvento, LIBXML_NOBLANKS);
            //$domXml = $this->removerTagsVazias($domXml); //opcional
            $domXml->formatOutput = true;
            $xmlTemp = $domXml->saveXML($domXml->documentElement);

            $domXml = new \DOMDocument("1.0", "UTF-8");
            $domXml->loadXML($xmlTemp);
            $domXml->encoding = "UTF-8";

            /* Busca qual tag será assinada */
            $tagAssinatura = "";
            $id = "";
            /** @var \DOMNode $nodeEvento */
            $nodeEvento = $domXml->getElementsByTagName('eSocial')->item(0)->childNodes[1]; //Pega o nome da tag principal do evento
            //$id = $nodeEvento->getAttribute("Id");
            $tagAssinatura = $nodeEvento->nodeName;
            /** @var \DOMNode $node */
            /*foreach ($nodeEvento->childNodes as $node) {
                if ($node->nodeType == 1 && ($node->nodeName != 'ideEvento' && $node->nodeName != 'ideEmpregador')) {
                    $tagAssinatura = $node->nodeName;
                    $node->setAttribute("Id", $id);
                }
            }*/

            foreach (array($tagAssinatura) as $tag) {
                $domXmlAssinado = $this->assinaXmlAux($domXml, $tag, $chavePublica, $chavePrivada);
            }
            
            $xmlAssinado = $domXmlAssinado->saveXML($domXmlAssinado->documentElement);

            return $xmlAssinado;
        } catch(\Exception $e) {
            throw $e;
        }
    }

    public function validaSchemaXml($xmlContent) {
        try {
            $validation = '';
            $schema = $this->schemaDir . DIRECTORY_SEPARATOR . $this->schemaXsd;

            if (file_exists($schema)) {
                $xml = new \DOMDocument("1.0", "UTF-8");
                $xml->loadXML($xmlContent, LIBXML_NOBLANKS);

                // Enable user error handling
                libxml_use_internal_errors(true);

                if (!$xml->schemaValidate($schema)) {
                    $validation = 'Erros encontrados:';

                    $errors = libxml_get_errors();
                    foreach ($errors as $error) {
                        $errorMsg = "\n";
                        /*switch ($error->level) {
                            case LIBXML_ERR_WARNING:
                                $errorMsg .= "Warning $error->code: ";
                                break;
                            case LIBXML_ERR_ERROR:
                                $errorMsg .= "Error $error->code: ";
                                break;
                            case LIBXML_ERR_FATAL:
                                $errorMsg .= "Fatal Error $error->code: ";
                                break;
                        }*/

                        // Retira a descrição do schema da mensagem de erro (primeira ocorrencia entre chaves)
                        $posIni = strpos($error->message, '{');
                        $posFim = strpos($error->message, '}');
                        $errorMsgSchema = substr($error->message, $posIni, $posFim-$posIni+1);
                        $errorMsg .= str_replace($errorMsgSchema, '', trim($error->message));

                        /*if ($error->file) {
                            $errorMsg .= " in $error->file";
                        }*/
                        //$errorMsg .= " on line $error->line";
                        $validation .= $errorMsg;
                    }
                    libxml_clear_errors();
                } else {
                    $validation = 'Não foram encontrados erros';
                }
            }
            return $validation;
        } catch(\Exception $e) {
            throw $e;
        }
    }

}