<?php
namespace HCM\Geral\Repository;

use HCM\Util\HCMParameters;
use Doctrine\ORM\EntityManager;

class CalendarioRepository {
    
    protected $entityManager;
    protected $hcmParameters;
    
    public function __construct(EntityManager $entityManager, HCMParameters $hcmParameters) {
        $this->entityManager = $entityManager;
        $this->hcmParameters = $hcmParameters;
    }
    
    //Query que utiliza o numero do vinculo para verificar o calendario.
    public function getFeriadosVinculo($nrvinculo) {
        
        $sql = "SELECT
    VM.NRVINCULOM,
    VM.NMVINCULOM,
    VH.DTMESCOMPETENC AS DTMESCOMPETENC_VINC,
    VH.NRESTRUTLEGAL,
    VH.NRESTRUTGEREN,
    ESTRUT_CALEND.NRESTRUTURAM,
    ESTRUT_CALEND.NMESTRUTURAM,
    TO_CHAR(FER.DTCALENDARIO, 'DD/MM/YYYY') DTCALENDARIO,
    FER.DSOBSCALEND
FROM GPE_VINCULOM VM
JOIN GPE_VINCULOH VH ON VH.NRORG = VM.NRORG AND VH.NRVINCULOM = VM.NRVINCULOM AND VH.DTMESCOMPETENC = (
    SELECT MAX(DTMESCOMPETENC) FROM GPE_VINCULOH WHERE NRORG = VH.NRORG AND NRVINCULOM = VH.NRVINCULOM AND DTMESCOMPETENC <= :P_DTMESCOMPETENC
)
JOIN ESTRUTURAM ESTRUT_CALEND ON ESTRUT_CALEND.NRORG = VM.NRORG
    AND ESTRUT_CALEND.NRESTRUTURAM = (
        SELECT
            MOV.NRESTRUTURAM
        FROM GPE_MOVIMENTACAO MOV
        JOIN GPE_PARAMORG PO ON PO.NRORG = MOV.NRORG
        WHERE MOV.NRORG = ESTRUT_CALEND.NRORG
            AND MOV.NRVINCULOM = VM.NRVINCULOM
            AND MOV.DTINIMOVIMENT <= LAST_DAY(:P_DTMESCOMPETENC)
            AND (MOV.DTFIMMOVIMENT IS NULL OR MOV.DTFIMMOVIMENT > LAST_DAY(:P_DTMESCOMPETENC))
            AND MOV.NRTIPOESTRUTURA IN (NVL(PO.NRTPESTRUTCALEND, 0), PO.NRTPESTRUTLEGAL)
        ORDER BY DECODE(MOV.NRTIPOESTRUTURA, PO.NRTPESTRUTCALEND, 1, PO.NRTPESTRUTLEGAL, 2, 3), MOV.DTINIMOVIMENT DESC
        FETCH FIRST ROW ONLY
    )
JOIN ESTRUTURAH ESTRUT_CALENDH ON ESTRUT_CALENDH.NRORG = VM.NRORG AND ESTRUT_CALENDH.NRESTRUTURAM = ESTRUT_CALEND.NRESTRUTURAM
    AND ESTRUT_CALENDH.DTMESCOMPETENC = (
        SELECT MAX(DTMESCOMPETENC) FROM ESTRUTURAH WHERE NRORG = ESTRUT_CALENDH.NRORG AND NRESTRUTURAM = ESTRUT_CALENDH.NRESTRUTURAM AND DTMESCOMPETENC <= :P_DTMESCOMPETENC
    )
JOIN OCORCALENDARIO FER ON FER.NRORG = VM.NRORG AND FER.NRCALENDARIO = ESTRUT_CALENDH.NRCALENDARIO
    /* GARANTE QUE APENAS OS FERIADOS EXISTENTES APOS A ADMISSAO DO VINCULO SEJAM CONSIDERADOS */
    AND FER.DTCALENDARIO >= VM.DTADMISSAOVINC
WHERE VM.NRORG = :P_NRORG
    AND VM.NRVINCULOM = :P_NRVINCULOM";
    
        $params = array('P_NRORG' => $this->hcmParameters->getNrorgTrabalho(), 'P_NRVINCULOM' => $nrvinculo, 'P_DTMESCOMPETENC' => $this->hcmParameters->getDtCompetencia());
        
        $query = $this->entityManager->getConnection()->prepare($sql);
        $query->execute($params);
        
        return $query->fetchAll();
        
    }
    
}

