<?php
namespace HCM\Geral\Repository;

use Doctrine\DBAL\Connection;
use Doctrine\ORM\EntityManager;

class PreFechamentoFgtsRepository {
    
    public function __construct(Connection $connection, EntityManager $entityManager) {
        $this->connection = $connection;
        $this->entityManager = $entityManager;
        $this->conferenciaFgts = new ConferenciaFgts($entityManager);
        $this->informacoesSinteticasFgts = new InformacoesSinteticasFgts($entityManager);
        $this->informacoesSobreBasesCalculoFgts = new InformacoesSobreBasesCalculoFgts($entityManager);
    }
        
    public function getConferenciaFgts($parameters) {
        return $this->conferenciaFgts->getConferenciaFgts($parameters);
    }
    
    public function getInformacoesSinteticasFgts($parameters) {
        return $this->informacoesSinteticasFgts->getInformacoesSinteticasFgts($parameters);
    }
    
    public function getInformacoesSobreBasesCalculoFgts($parameters) {
        return $this->informacoesSobreBasesCalculoFgts->getInformacoesSobreBasesCalculoFgts($parameters);
    }
}

class ConferenciaFgts {
    
    public function __construct(EntityManager $entityManager) {
        $this->entityManager = $entityManager;
    }
    
    public function getConferenciaFgts($parameters) {
        $sql = $this->entityManager->getConnection()->prepare(Queries::$conferenciaFgts);
        $sql->execute(
            array(
                'P_NRORG'          => $parameters['NRORG'],
                'P_MODALIDADECAL'  => 1,
                'P_COMPETENCIA'    => $parameters['DTMESCOMPETENC'],
                'P_NRESTRUTLEGAL'  => is_array($parameters['NRESTRUTLEGAL']) ? implode(",",$parameters['NRESTRUTLEGAL']) : $parameters['NRESTRUTLEGAL'],
                'P_NRVINCULO'      => '',
                'P_NRTABELA'       => $parameters['NRTABELA']
            )
        );
        return $sql->fetchAll();
    }
}

class InformacoesSinteticasFgts {
    
    public function __construct(EntityManager $entityManager) {
        $this->entityManager = $entityManager;
    }
    
    public function getInformacoesSinteticasFgts($parameters) {
        $sql = $this->entityManager->getConnection()->prepare(Queries::$informacoesSinteticasFgts);
        $sql->execute(
            array(
                'P_NRORG'         => $parameters['NRORG'],
                'P_MODALIDADECAL' => 1,
                'P_COMPETENCIA'   => $parameters['DTMESCOMPETENC'],
                'P_NRESTRUTLEGAL' => is_array($parameters['NRESTRUTLEGAL']) ? implode(",",$parameters['NRESTRUTLEGAL']) : $parameters['NRESTRUTLEGAL'],
                'P_NRVINCULO'     => '',
                'P_NRTABELA'      => $parameters['NRTABELA']
            )
        );
        return $sql->fetchAll();
    }
}

class InformacoesSobreBasesCalculoFgts {
    
    public function __construct(EntityManager $entityManager) {
        $this->entityManager = $entityManager;
    }
    
    public function getInformacoesSobreBasesCalculoFgts($parameters) {
        $sql = $this->entityManager->getConnection()->prepare(Queries::$informacoesSobreBasesCalculoFgts);
        $sql->execute(
            array(
                'P_NRORG'         => $parameters['NRORG'],
                'P_COMPETENCIA'   => $parameters['DTMESCOMPETENC'],
                'P_NRESTRUTLEGAL' => is_array($parameters['NRESTRUTLEGAL']) ? implode(",",$parameters['NRESTRUTLEGAL']) : $parameters['NRESTRUTLEGAL'],
                'P_NRVINCULO'     => '',
                'P_NRTABELA'      => $parameters['NRTABELA']
            )
        );
        return $sql->fetchAll();
    }
}

class Queries {
    
    static $conferenciaFgts = "
      SELECT * FROM ( 
        SELECT RES.VINCULO,
              RES.DTRESCISAOVINC,
              RES.MATRICULA,
              RES.REGISTRO,
              PE.NRCPFPESSOA,
              RES.NOME,
              RES.NRLEGAL,
              RES.NRGEREN,
              RES.RAZAOLEGAL,
              RES.CNPJLEGAL,
              RES.INTLEGAL,
              RES.RAZAOGEREN,
              RES.CNPJGEREN,
              RES.INTGEREN,
              NVL(ESOPE.IDSTATUS, ESONP.IDSTATUS) IDSTATUS,
              NVL(ESOPE.NRTABELA, ESONP.NRTABELA) NRTABELA,
              :P_COMPETENCIA PERIODO,
              SUBSTR(RES.CNPJLEGAL, 0,8) CNPJRAIZ,
              RES.CDESOCIAL CDCATEGESOC,
              (RES.CDESOCIAL || ' - ' || RES.DSCATEGESOC) DSCATEGESOC,
              RES.BASE BASE_FGTS,
              RES.FGTS DEPOSITO_FGTS,
              SUM(NVL(ESONP.VRBASEFGTSMENSAL,0) + NVL(ESOPE.VRBASEFGTSMENSAL,0) + NVL(ESONP.VRBASEFGTS13,0) + NVL(ESOPE.VRBASEFGTS13,0)) BASE_FGTS_ESOCIAL,
              SUM(NVL(ESONP.VRDEPOSITOFGTS,0) + NVL(ESOPE.VRDEPOSITOFGTS,0) + NVL(ESONP.VRDEPOSITOFGTS13,0) + NVL(ESOPE.VRDEPOSITOFGTS13,0)) DEPOSITO_FGTS_ESOCIAL,
              RES.BASE - SUM(NVL(ESONP.VRBASEFGTSMENSAL,0) + NVL(ESOPE.VRBASEFGTSMENSAL,0) + NVL(ESONP.VRBASEFGTS13,0) + NVL(ESOPE.VRBASEFGTS13,0)) DIFERENCA_BASE,
              RES.FGTS - SUM(NVL(ESONP.VRDEPOSITOFGTS,0) + NVL(ESOPE.VRDEPOSITOFGTS,0) + NVL(ESONP.VRDEPOSITOFGTS13,0) + NVL(ESOPE.VRDEPOSITOFGTS13,0)) DIFERENCA_DEPOSITO
          FROM (SELECT VM.NRORG, VM.NRPESSOA,
                      VM.NRVINCULOM      VINCULO,
                      VM.DTRESCISAOVINC,
                      VM.CDMATRICULA     MATRICULA,
                      VM.DSLIVROADMISSAO REGISTRO,
                      VM.NMVINCULOM      NOME,
                      VM.NRTIPOVINCULOM,
                      SUM(CASE WHEN ICF.NREVENTOM IN (3060, 3180, 3210,  3120) THEN ICF.VREVENTOFOLHA ELSE 0 END) BASE,
                      SUM(CASE WHEN ICF.NREVENTOM IN (2010, 2050, 2080, 2020) THEN ICF.VREVENTOFOLHA ELSE 0 END) FGTS,
                      CF.DTOCORRENCIA,
                      VH.NRESTRUTLEGAL   NRLEGAL,
                      VH.NRESTRUTGEREN   NRGEREN,
                      EHL.NMRAZSOCESTRUT RAZAOLEGAL,
                      EHL.CDCNPJESTRUT   CNPJLEGAL,
                      EML.CDINTESTRUTURA INTLEGAL,
                      EHG.NMRAZSOCESTRUT RAZAOGEREN,
                      EHG.CDCNPJESTRUT   CNPJGEREN,
                      EMG.CDINTESTRUTURA INTGEREN,
                      CATEGTRAB.DSCATEGESOC, 
                      CATEGTRAB.CDESOCIAL, 
                      EP.NRTPAMBIENTE
                  FROM GPE_VINCULOM VM
                  JOIN GPE_VINCULOH VH ON VH.NRORG = VM.NRORG
                                      AND VH.NRVINCULOM = VM.NRVINCULOM
                                      AND VH.DTMESCOMPETENC = ( SELECT MAX(DTMESCOMPETENC) 
                                                                  FROM GPE_VINCULOH
                                                                WHERE DTMESCOMPETENC <= :P_COMPETENCIA
                                                                  AND NRORG      = VM.NRORG
                                                                  AND NRVINCULOM = VM.NRVINCULOM )
                  /*ESTRUTURA LEGAL*/
                  JOIN ESTRUTURAM EML ON EML.NRORG = VH.NRORG
                                    AND EML.NRESTRUTURAM = VH.NRESTRUTLEGAL
                  LEFT JOIN ESTRUTURAH EHL ON EHL.NRORG = VH.NRORG
                                          AND EHL.NRESTRUTURAM = EML.NRESTRUTURAM
                                          AND EHL.DTMESCOMPETENC = ( SELECT MAX(DTMESCOMPETENC) 
                                                                      FROM ESTRUTURAH
                                                                      WHERE DTMESCOMPETENC <= :P_COMPETENCIA
                                                                        AND NRORG = VH.NRORG
                                                                        AND NRESTRUTURAM = EML.NRESTRUTURAM )
                  /*ESTRUTURA GERENCIAL*/
                  JOIN ESTRUTURAM EMG ON EMG.NRORG = VH.NRORG
                                    AND EMG.NRESTRUTURAM = VH.NRESTRUTGEREN
                  LEFT JOIN ESTRUTURAH EHG ON EHG.NRORG = VH.NRORG
                                          AND EHG.NRESTRUTURAM = EMG.NRESTRUTURAM
                                          AND EHG.DTMESCOMPETENC = ( SELECT MAX(DTMESCOMPETENC) 
                                                                      FROM ESTRUTURAH
                                                                      WHERE DTMESCOMPETENC <= :P_COMPETENCIA
                                                                        AND NRORG        = VH.NRORG
                                                                        AND NRESTRUTURAM = EMG.NRESTRUTURAM )
                  /*CÁLCULO DE FOLHA*/
                  JOIN FPA_ITECALCFOLHA ICF ON ICF.NRORG = VM.NRORG
                                          AND ICF.NRVINCULOM  = VM.NRVINCULOM
                                          AND ICF.NREVENTOM IN (3060, 2010, 3180, 2050, 3210, 2080, 3120, 2020)
                                          AND EXISTS (SELECT 1 FROM FPA_ITECALCFOLHA IC
                                                       WHERE IC.NRORG = ICF.NRORG
                                                         AND IC.NRCALCULOFOLHA = ICF.NRCALCULOFOLHA
                                                         AND IC.NRVINCULOM = ICF.NRVINCULOM
                                                         AND IC.NREVENTOM IN (3030, 3050))
                  JOIN FPA_CALCULOFOLHA CF ON CF.NRORG = ICF.NRORG
                                          AND CF.NRCALCULOFOLHA = ICF.NRCALCULOFOLHA
                                          AND ((CF.NRTIPOMOVIMENT IN (1,2,10,19) AND :P_NRTABELA = 1) OR
                                             (CF.NRTIPOMOVIMENT = 4 AND :P_NRTABELA = 4) OR
                                             (CF.NRTIPOMOVIMENT IN (1,2,4,10,19) AND :P_NRTABELA IS NULL))
                                          AND CF.NRTPMODALIDCAL = :P_MODALIDADECAL
                                          AND CF.DTOCORRENCIA = :P_COMPETENCIA
                  /*CATEGORIA*/
                  JOIN GPE_VINCULOEMPREG VINCEMPREG ON VINCEMPREG.NRVINCULOEMPREG = VM.NRVINCULOEMPREG 
                  JOIN GPE_CATEGTRABESOC CATEGTRAB ON CATEGTRAB.NRCATEGESOC = VINCEMPREG.NRCATEGESOC 
                  JOIN ESO_PARAMETRO EP ON EP.NRORG = EML.NRORG 
                    AND EP.NRESTRUTURAM = EML.NRESTRUTURAM 
                  /**/
                WHERE VM.NRORG = :P_NRORG
                  AND ( INSTR(','||:P_NRVINCULO||',', ','||VM.NRVINCULOM||',' ) > 0 OR :P_NRVINCULO IS NULL ) --[15-02-12][multisearch_vinculo_156497]
                  AND ( VM.DTADMISSAOVINC <= LAST_DAY(:P_COMPETENCIA) )
                  AND ( VM.DTRESCISAOVINC >= :P_COMPETENCIA OR VM.DTRESCISAOVINC IS NULL OR 1 = 1)
                  AND ( INSTR(','||:P_NRESTRUTLEGAL||',', ','||VH.NRESTRUTLEGAL||',' ) > 0 OR :P_NRESTRUTLEGAL IS NULL )
                GROUP BY VM.NRORG, VM.NRPESSOA, VM.NRVINCULOM,
                  VM.DTRESCISAOVINC,
                  VM.CDMATRICULA,
                  VM.DSLIVROADMISSAO,
                  VM.NMVINCULOM,
                  VM.NRTIPOVINCULOM,
                  /*ICF.NREVENTOM,*/
                  CF.DTOCORRENCIA,
                  VH.NRESTRUTLEGAL,
                  VH.NRESTRUTGEREN,
                  EHL.NMRAZSOCESTRUT,
                  EHL.CDCNPJESTRUT,
                  EML.CDINTESTRUTURA,
                  EHG.NMRAZSOCESTRUT,
                  EHG.CDCNPJESTRUT,
                  EMG.CDINTESTRUTURA,
                  CATEGTRAB.DSCATEGESOC,
                  CATEGTRAB.CDESOCIAL,
                  EP.NRTPAMBIENTE
                ) RES
        LEFT JOIN (SELECT ESONP.DTMESCOMPETENC,
                    ESONP.NRTPAMBIENTE,
                    ESONP.NRVINCULOM,
                    ESONP.NRESTRUTURAM,
                    ESONP.NRORG,
                    ESONP.IDSTATUS,
                    ESONP.NRTABELA,
                    SUM(NVL(NPFGTS.VRBASEFGTSMENSAL,0)) AS VRBASEFGTSMENSAL,
                    SUM(NVL(NPFGTS.VRBASEFGTS13,0)) AS VRBASEFGTS13,
                    SUM(NVL(NPFGTS.VRDEPOSITOFGTS, 0)) AS VRDEPOSITOFGTS,
                    SUM(NVL(NPFGTS.VRDEPOSITOFGTS13,0)) AS VRDEPOSITOFGTS13,
                    ESONP.NRENVIO,
                    EHPN.CDCNPJESTRUT
                FROM ESO_NAOPERIODICO ESONP 
                JOIN ESTRUTURAH EHPN ON EHPN.NRORG   = ESONP.NRORG
                                          AND EHPN.NRESTRUTURAM = ESONP.NRESTRUTURAM
                                          AND EHPN.DTMESCOMPETENC = ( SELECT MAX(DTMESCOMPETENC) FROM ESTRUTURAH
                                                                      WHERE DTMESCOMPETENC <= :P_COMPETENCIA
                                                                      AND NRORG        = ESONP.NRORG
                                                                      AND NRESTRUTURAM = EHPN.NRESTRUTURAM )
                JOIN ESO_NAOPERIODFGTS NPFGTS ON NPFGTS.NRORG = ESONP.NRORG
                                        AND NPFGTS.NRENVIO = ESONP.NRENVIO
                WHERE ESONP.NRORG = :P_NRORG
                AND ESONP.IDATIVO = 'S' AND ESONP.NRENVIOEXCL IS NULL
                AND ESONP.IDSTATUS IN('C','A')
                AND (ESONP.NRTABELA = :P_NRTABELA OR (:P_NRTABELA = 1 AND ESONP.NRTABELA IS NULL) OR :P_NRTABELA IS NULL)
                AND ESONP.NMEVENTO IN ('S-2299','S-2399')
                GROUP BY ESONP.DTMESCOMPETENC, ESONP.NRTPAMBIENTE, ESONP.NRVINCULOM,ESONP.NRENVIO, EHPN.CDCNPJESTRUT , ESONP.NRESTRUTURAM , ESONP.NRORG, ESONP.IDSTATUS, ESONP.NRTABELA
            )ESONP ON TRUNC(ESONP.DTMESCOMPETENC, 'MM') = RES.DTOCORRENCIA
                   AND ESONP.NRTPAMBIENTE = RES.NRTPAMBIENTE
                   AND ESONP.NRVINCULOM = RES.VINCULO
                   AND SUBSTR(ESONP.CDCNPJESTRUT,1,8) = SUBSTR(RES.CNPJLEGAL,1,8)
                   
        LEFT JOIN (SELECT ESOPE.DTMESCOMPETENC,
                ESOPE.NRTPAMBIENTE,
                ESOPE.NRCHAVE,
                ESOPE.NRESTRUTURAM,
                ESOPE.NRORG,
                ESOPE.IDSTATUS,
                ESOPE.NRTABELA,
                SUM(NVL(PEFGTS.VRBASEFGTSMENSAL,0)) AS VRBASEFGTSMENSAL,
                SUM(NVL(PEFGTS.VRBASEFGTS13,0)) AS VRBASEFGTS13,
                SUM(NVL(PEFGTS.VRDEPOSITOFGTS, 0)) AS VRDEPOSITOFGTS,
                SUM(NVL(PEFGTS.VRDEPOSITOFGTS13,0)) AS VRDEPOSITOFGTS13,
                ESOPE.NRENVIO,
                EHPE.CDCNPJESTRUT 
                FROM ESO_PERIODICO ESOPE   
                JOIN ESTRUTURAH EHPE ON EHPE.NRORG   = ESOPE.NRORG
                                          AND EHPE.NRESTRUTURAM = ESOPE.NRESTRUTURAM
                                          AND EHPE.DTMESCOMPETENC = ( SELECT MAX(DTMESCOMPETENC) FROM ESTRUTURAH
                                                                      WHERE DTMESCOMPETENC <= :P_COMPETENCIA
                                                                      AND NRORG        = ESOPE.NRORG
                                                                      AND NRESTRUTURAM = EHPE.NRESTRUTURAM )
                JOIN ESO_PERIODFGTS PEFGTS ON PEFGTS.NRORG = ESOPE.NRORG
                                        AND PEFGTS.NRENVIO = ESOPE.NRENVIO
                WHERE ESOPE.NRORG = :P_NRORG
                AND ESOPE.IDATIVO = 'S' AND ESOPE.NRENVIOEXCL IS NULL
                AND ESOPE.IDSTATUS IN('C','A')
                AND (ESOPE.NRTABELA = :P_NRTABELA OR (:P_NRTABELA = 1 AND ESOPE.NRTABELA IS NULL) OR :P_NRTABELA IS NULL)
                AND ESOPE.NMEVENTO = 'S-1200'
                GROUP BY ESOPE.DTMESCOMPETENC,ESOPE.NRTPAMBIENTE,ESOPE.NRCHAVE,ESOPE.NRENVIO, EHPE.CDCNPJESTRUT ,ESOPE.NRESTRUTURAM, ESOPE.NRORG, ESOPE.IDSTATUS, ESOPE.NRTABELA
            )ESOPE ON TRUNC(ESOPE.DTMESCOMPETENC, 'MM') = RES.DTOCORRENCIA
                   AND ESOPE.NRTPAMBIENTE = RES.NRTPAMBIENTE
                   AND ESOPE.NRCHAVE = RES.NRPESSOA
                   AND SUBSTR(ESOPE.CDCNPJESTRUT,1,8) = SUBSTR(RES.CNPJLEGAL,1,8)
        JOIN GPE_PESSOAH PE ON PE.NRPESSOA = RES.NRPESSOA 
                          AND PE.NRORG = RES.NRORG 
                          AND PE.DTMESCOMPETENC = ( SELECT MAX(DTMESCOMPETENC) 
                                                      FROM GPE_PESSOAH 
                                                      WHERE DTMESCOMPETENC <= :P_COMPETENCIA
                                                        AND NRORG = PE.NRORG
                                                        AND NRPESSOA = PE.NRPESSOA )
      GROUP BY RES.VINCULO,
                RES.DTRESCISAOVINC,
                RES.MATRICULA,
                RES.REGISTRO,
                PE.NRCPFPESSOA,
                RES.NOME,
                RES.NRLEGAL,
                RES.NRGEREN,
                RES.RAZAOLEGAL,
                RES.CNPJLEGAL,
                RES.INTLEGAL,
                RES.RAZAOGEREN,
                RES.CNPJGEREN,
                RES.INTGEREN,
                NVL(ESOPE.IDSTATUS, ESONP.IDSTATUS),
                NVL(ESOPE.NRTABELA, ESONP.NRTABELA),
                RES.CDESOCIAL,
                (RES.CDESOCIAL || ' - ' || RES.DSCATEGESOC),
                RES.BASE,
                RES.FGTS
      )
      ORDER BY CNPJRAIZ, CNPJLEGAL, NOME";
    
    static $informacoesSinteticasFgts = "
      SELECT 
      * 
      FROM (
        SELECT 
            NRLEGAL,
            RAZAOLEGAL,
            ('('||NRLEGAL||') '||RAZAOLEGAL) ESTABELECIMENTO,
            CNPJLEGAL,
            INTLEGAL,
            CNPJRAIZ,
            SUM(DEPOSITO_FGTS) DEPOSITO_FGTS,
            SUM(DEPOSITO_FGTS_ESOCIAL) DEPOSITO_FGTS_ESOCIAL,
            SUM(DEPOSITO_FGTS_13) DEPOSITO_FGTS_13,
            SUM(DEPOSITO_FGTS_ESOCIAL_13) DEPOSITO_FGTS_ESOCIAL_13,
            /*SUM(FGTS_RESCISAO) FGTS_RESCISAO,
            SUM( (CASE WHEN DTRESCISAOVINC IS NOT NULL THEN DEPOSITO_FGTS_ESOCIAL ELSE 0 END) ) RESCISAO_ESOCIAL,*/
            SUM((DEPOSITO_FGTS + DEPOSITO_FGTS_13) - (DEPOSITO_FGTS_ESOCIAL + DEPOSITO_FGTS_ESOCIAL_13)) DIFERENCA_DEPOSITO
          FROM (
              SELECT RES.VINCULO,
                    PE.NRCPFPESSOA,
                    RES.NOME,
                    RES.NRLEGAL,
                    RES.RAZAOLEGAL,
                    RES.CNPJLEGAL,
                    RES.INTLEGAL,
                    NVL(ESOPE.IDSTATUS, ESONP.IDSTATUS) IDSTATUS,
                    SUBSTR(RES.CNPJLEGAL, 0,8) CNPJRAIZ,
                    SUM(RES.FGTS_MES) DEPOSITO_FGTS,
                    SUM(RES.FGTS_13) DEPOSITO_FGTS_13,
                    SUM(NVL(ESONP.VRDEPOSITOFGTS,0) + NVL(ESOPE.VRDEPOSITOFGTS,0)) DEPOSITO_FGTS_ESOCIAL,
                    SUM(NVL(ESONP.VRDEPOSITOFGTS13,0) + NVL(ESOPE.VRDEPOSITOFGTS13,0)) DEPOSITO_FGTS_ESOCIAL_13
                    /*SUM(RES.FGTS_RESCISAO) FGTS_RESCISAO*/
                FROM (SELECT VM.NRORG, VM.NRPESSOA,
                            VM.NRVINCULOM      VINCULO,
                            VM.NMVINCULOM      NOME,
                            CF.DTOCORRENCIA,
                            VH.NRESTRUTLEGAL   NRLEGAL,
                            EHL.NMRAZSOCESTRUT RAZAOLEGAL,
                            EHL.CDCNPJESTRUT   CNPJLEGAL,
                            EML.CDINTESTRUTURA INTLEGAL,
                            EP.NRTPAMBIENTE,
                            SUM(CASE WHEN ICF.NREVENTOM IN (2010, 2050) THEN ICF.VREVENTOFOLHA ELSE 0 END) FGTS_MES,
                            SUM(CASE WHEN ICF.NREVENTOM IN (2020, 2080) THEN ICF.VREVENTOFOLHA ELSE 0 END) FGTS_13
                        FROM GPE_VINCULOM VM
                        JOIN GPE_VINCULOH VH ON VH.NRORG = VM.NRORG
                                            AND VH.NRVINCULOM = VM.NRVINCULOM
                                            AND VH.DTMESCOMPETENC = ( SELECT MAX(DTMESCOMPETENC) 
                                                                        FROM GPE_VINCULOH
                                                                      WHERE DTMESCOMPETENC <= :P_COMPETENCIA
                                                                        AND NRORG      = VM.NRORG
                                                                        AND NRVINCULOM = VM.NRVINCULOM )
                        /*ESTRUTURA LEGAL*/
                        JOIN ESTRUTURAM EML ON EML.NRORG = VH.NRORG
                                          AND EML.NRESTRUTURAM = VH.NRESTRUTLEGAL
                        LEFT JOIN ESTRUTURAH EHL ON EHL.NRORG = VH.NRORG
                                                AND EHL.NRESTRUTURAM = EML.NRESTRUTURAM
                                                AND EHL.DTMESCOMPETENC = ( SELECT MAX(DTMESCOMPETENC) 
                                                                            FROM ESTRUTURAH
                                                                            WHERE DTMESCOMPETENC <= :P_COMPETENCIA
                                                                              AND NRORG = VH.NRORG
                                                                              AND NRESTRUTURAM = EML.NRESTRUTURAM )
                      /*CÁLCULO DE FOLHA*/
                      JOIN FPA_ITECALCFOLHA ICF ON ICF.NRORG = VM.NRORG
                                              AND ICF.NRVINCULOM = VM.NRVINCULOM
                                              AND ICF.NREVENTOM IN (2010, 2050, 2080, 2020)
                                              AND EXISTS (SELECT 1 FROM FPA_ITECALCFOLHA IC
                                                           WHERE IC.NRORG = ICF.NRORG
                                                             AND IC.NRCALCULOFOLHA = ICF.NRCALCULOFOLHA
                                                             AND IC.NRVINCULOM = ICF.NRVINCULOM
                                                             AND IC.NREVENTOM IN (3030, 3050))
                      JOIN FPA_CALCULOFOLHA CF ON CF.NRORG = ICF.NRORG
                                              AND CF.NRCALCULOFOLHA = ICF.NRCALCULOFOLHA
                                              AND ((CF.NRTIPOMOVIMENT IN (1,2,10,19) AND :P_NRTABELA = 1) OR
                                                 (CF.NRTIPOMOVIMENT = 4 AND :P_NRTABELA = 4) OR
                                                 (CF.NRTIPOMOVIMENT IN (1,2,4,10,19) AND :P_NRTABELA IS NULL))
                                              AND CF.NRTPMODALIDCAL = :P_MODALIDADECAL
                                              AND CF.DTOCORRENCIA = :P_COMPETENCIA
                      JOIN ESO_PARAMETRO EP ON EP.NRORG = EML.NRORG 
                        AND EP.NRESTRUTURAM = EML.NRESTRUTURAM 
                      WHERE VM.NRORG = :P_NRORG
                        AND ( INSTR(','||:P_NRVINCULO||',', ','||VM.NRVINCULOM||',' ) > 0 OR :P_NRVINCULO IS NULL ) --[15-02-12][multisearch_vinculo_156497]
                        AND ( VM.DTADMISSAOVINC <= LAST_DAY(:P_COMPETENCIA) )
                        AND ( VM.DTRESCISAOVINC >= :P_COMPETENCIA OR VM.DTRESCISAOVINC IS NULL OR 1 = 1)
                        AND ( INSTR(','||:P_NRESTRUTLEGAL||',', ','||VH.NRESTRUTLEGAL||',' ) > 0 OR :P_NRESTRUTLEGAL IS NULL )
                      GROUP BY VM.NRORG, VM.NRPESSOA, VM.NRVINCULOM,
                        VM.NMVINCULOM,
                        /*ICF.NREVENTOM,*/
                        CF.DTOCORRENCIA,
                        VH.NRESTRUTLEGAL,
                        EHL.NMRAZSOCESTRUT,
                        EHL.CDCNPJESTRUT,
                        EML.CDINTESTRUTURA,
                        EP.NRTPAMBIENTE
                      ) RES
          LEFT JOIN (SELECT ESONP.DTMESCOMPETENC,
                    ESONP.NRTPAMBIENTE,
                    ESONP.NRVINCULOM,
                    ESONP.IDSTATUS,
                    EHPN.CDCNPJESTRUT,
                    SUM(NVL(NPFGTS.VRDEPOSITOFGTS,0)) AS VRDEPOSITOFGTS,
                    SUM(NVL(NPFGTS.VRDEPOSITOFGTS13,0)) AS VRDEPOSITOFGTS13
                  FROM ESO_NAOPERIODICO ESONP 
                  JOIN ESTRUTURAH EHPN ON EHPN.NRORG   = ESONP.NRORG
                                            AND EHPN.NRESTRUTURAM = ESONP.NRESTRUTURAM
                                            AND EHPN.DTMESCOMPETENC = ( SELECT MAX(DTMESCOMPETENC) FROM ESTRUTURAH
                                                                        WHERE DTMESCOMPETENC <= :P_COMPETENCIA
                                                                        AND NRORG        = ESONP.NRORG
                                                                        AND NRESTRUTURAM = EHPN.NRESTRUTURAM )
                  JOIN ESO_NAOPERIODFGTS NPFGTS ON NPFGTS.NRORG = ESONP.NRORG
                                          AND NPFGTS.NRENVIO = ESONP.NRENVIO
                  WHERE ESONP.NRORG = :P_NRORG
                  AND ESONP.IDATIVO = 'S' AND ESONP.NRENVIOEXCL IS NULL
                  AND ESONP.IDSTATUS IN('C','A')
                  AND (ESONP.NRTABELA = :P_NRTABELA OR (:P_NRTABELA = 1 AND ESONP.NRTABELA IS NULL) OR :P_NRTABELA IS NULL)
                  AND ESONP.NMEVENTO IN ('S-2299','S-2399')
                  GROUP BY ESONP.DTMESCOMPETENC, ESONP.NRTPAMBIENTE, ESONP.NRVINCULOM, ESONP.IDSTATUS, EHPN.CDCNPJESTRUT
                )ESONP ON TRUNC(ESONP.DTMESCOMPETENC, 'MM') = RES.DTOCORRENCIA
                     AND ESONP.NRTPAMBIENTE = RES.NRTPAMBIENTE
                     AND ESONP.NRVINCULOM = RES.VINCULO
                     AND SUBSTR(ESONP.CDCNPJESTRUT,1,8) = SUBSTR(RES.CNPJLEGAL,1,8)
                   
          LEFT JOIN (SELECT ESOPE.DTMESCOMPETENC,
                            ESOPE.NRTPAMBIENTE,
                            ESOPE.NRCHAVE,
                            EHPE.CDCNPJESTRUT,
                            ESOPE.IDSTATUS,
                            SUM(NVL(PEFGTS.VRDEPOSITOFGTS,0)) AS VRDEPOSITOFGTS,
                            SUM(NVL(PEFGTS.VRDEPOSITOFGTS13,0)) AS VRDEPOSITOFGTS13
                    FROM ESO_PERIODICO ESOPE   
                    JOIN ESTRUTURAH EHPE ON EHPE.NRORG   = ESOPE.NRORG
                                              AND EHPE.NRESTRUTURAM = ESOPE.NRESTRUTURAM
                                              AND EHPE.DTMESCOMPETENC = ( SELECT MAX(DTMESCOMPETENC) FROM ESTRUTURAH
                                                                          WHERE DTMESCOMPETENC <= :P_COMPETENCIA
                                                                          AND NRORG        = ESOPE.NRORG
                                                                          AND NRESTRUTURAM = EHPE.NRESTRUTURAM )
                    JOIN ESO_PERIODFGTS PEFGTS ON PEFGTS.NRORG = ESOPE.NRORG
                                            AND PEFGTS.NRENVIO = ESOPE.NRENVIO
                    WHERE ESOPE.NRORG = :P_NRORG
                    AND ESOPE.IDATIVO = 'S' AND ESOPE.NRENVIOEXCL IS NULL
                    AND ESOPE.IDSTATUS IN('C','A')
                    AND (ESOPE.NRTABELA = :P_NRTABELA OR (:P_NRTABELA = 1 AND ESOPE.NRTABELA IS NULL) OR :P_NRTABELA IS NULL)
                    AND ESOPE.NMEVENTO = 'S-1200'
                    GROUP BY ESOPE.DTMESCOMPETENC, ESOPE.NRTPAMBIENTE, ESOPE.NRCHAVE, EHPE.CDCNPJESTRUT, ESOPE.IDSTATUS
                  )ESOPE ON TRUNC(ESOPE.DTMESCOMPETENC, 'MM') = RES.DTOCORRENCIA
                       AND ESOPE.NRTPAMBIENTE = RES.NRTPAMBIENTE
                       AND ESOPE.NRCHAVE = RES.NRPESSOA
                       AND SUBSTR(ESOPE.CDCNPJESTRUT,1,8) = SUBSTR(RES.CNPJLEGAL,1,8)
              JOIN GPE_PESSOAH PE ON PE.NRPESSOA = RES.NRPESSOA 
                                AND PE.NRORG = RES.NRORG 
                                AND PE.DTMESCOMPETENC = ( SELECT MAX(DTMESCOMPETENC) 
                                                            FROM GPE_PESSOAH 
                                                            WHERE DTMESCOMPETENC <= :P_COMPETENCIA
                                                              AND NRORG = PE.NRORG
                                                              AND NRPESSOA = PE.NRPESSOA )
            GROUP BY RES.VINCULO,
                    PE.NRCPFPESSOA,
                    RES.NOME,
                    RES.NRLEGAL,
                    RES.RAZAOLEGAL,
                    RES.CNPJLEGAL,
                    RES.INTLEGAL,
                    NVL(ESOPE.IDSTATUS, ESONP.IDSTATUS),
                    SUBSTR(RES.CNPJLEGAL, 0,8)
            )
    GROUP BY NRLEGAL, RAZAOLEGAL, CNPJLEGAL, INTLEGAL, CNPJRAIZ)
    ORDER BY CNPJRAIZ, CNPJLEGAL";
    
    static $informacoesSobreBasesCalculoFgts = "
      SELECT * FROM ( 
        SELECT 
            NOME, 
            NRCPFPESSOA,
            NRLEGAL,
            RAZAOLEGAL,
            ('('||NRLEGAL||') '||RAZAOLEGAL||' '||CNPJLEGAL) ESTABELECIMENTO,
            CNPJLEGAL,
            PRI.CDTERCEIRO,
            CNPJRAIZ,
            PRI.VRBASEFGTSMENSAL BASE_ESOCIAL,
            PRI.VRBASEFGTS13 BASE_13_ESOCIAL,
            PRI.NRESTRUTLOTACAO,
            ( '('||PRI.NRESTRUTLOTACAO||') '||'FPAS '||PRI.CDFPAS||' '||'Terceiros '||PRI.CDTERCEIRO ) LOTACAO,
            (PRI.CDESOCIAL || ' - '|| PRI.DSCATEGESOC) DSCATEGESOC
          FROM (SELECT 
                    VM.NRVINCULOM VINCULO,
                    PE.NRCPFPESSOA,
                    VM.NMVINCULOM NOME,
                    VH.NRESTRUTLEGAL NRLEGAL,
                    EHL.NMRAZSOCESTRUT RAZAOLEGAL,
                    EHL.CDCNPJESTRUT CNPJLEGAL,
                    SUBSTR(EHL.CDCNPJESTRUT, 0,8) CNPJRAIZ,
                    SUM(NVL(NPFGTS.VRBASEFGTSMENSAL,0) + NVL(PEFGTS.VRBASEFGTSMENSAL,0) ) VRBASEFGTSMENSAL,
                    SUM(NVL(NPFGTS.VRBASEFGTS13,0) + NVL(PEFGTS.VRBASEFGTS13,0) ) VRBASEFGTS13,
                    NVL(ESOPE.NRESTRUTLOTACAO,ESONP.NRESTRUTLOTACAO) NRESTRUTLOTACAO,
                    NVL(LOT.CDFPAS,EHL.CDFPAS) CDFPAS,
                    NVL(LOT.CDTERCEIRO,EHL.CDTERCEIRO) CDTERCEIRO,
                    VM.DTRESCISAOVINC,
                    CATEGTRAB.DSCATEGESOC,
                    CATEGTRAB.CDESOCIAL
                  FROM GPE_VINCULOM VM
                  JOIN GPE_VINCULOH VH ON VH.NRORG = VM.NRORG
                                      AND VH.NRVINCULOM = VM.NRVINCULOM
                                      AND VH.DTMESCOMPETENC = ( SELECT MAX(DTMESCOMPETENC) FROM GPE_VINCULOH
                                                                WHERE DTMESCOMPETENC <= :P_COMPETENCIA
                                                                  AND NRORG      = VM.NRORG
                                                                  AND NRVINCULOM = VM.NRVINCULOM )
                  JOIN GPE_PESSOAH PE ON PE.NRPESSOA = VM.NRPESSOA 
                                    AND PE.NRORG = VM.NRORG 
                                    AND PE.DTMESCOMPETENC = (SELECT MAX(DTMESCOMPETENC) FROM GPE_PESSOAH 
                                                              WHERE DTMESCOMPETENC <= :P_COMPETENCIA
                                                                AND NRORG = PE.NRORG
                                                                AND NRPESSOA = PE.NRPESSOA)
                  /*ESTRUTURA LEGAL*/
                  JOIN ESTRUTURAM EML ON EML.NRORG = VH.NRORG
                                          AND EML.NRESTRUTURAM = VH.NRESTRUTLEGAL
                  LEFT JOIN ESTRUTURAH EHL ON EHL.NRORG        = VH.NRORG
                                          AND EHL.NRESTRUTURAM = EML.NRESTRUTURAM
                                          AND EHL.DTMESCOMPETENC = ( SELECT MAX(DTMESCOMPETENC) FROM ESTRUTURAH
                                                                    WHERE DTMESCOMPETENC <= :P_COMPETENCIA
                                                                      AND NRORG        = VH.NRORG
                                                                      AND NRESTRUTURAM = EML.NRESTRUTURAM )
                  JOIN ESO_PARAMETRO EP ON EP.NRORG = EML.NRORG
                                  AND EP.NRESTRUTURAM = EML.NRESTRUTURAM
                  LEFT JOIN ESO_NAOPERIODICO ESONP ON ESONP.NRORG = VM.NRORG
                                            AND TRUNC(ESONP.DTMESCOMPETENC, 'MM') = :P_COMPETENCIA
                                            AND ESONP.NRTPAMBIENTE = EP.NRTPAMBIENTE
                                            AND ESONP.IDATIVO = 'S' AND ESONP.NRENVIOEXCL IS NULL
                                            AND ESONP.IDSTATUS IN('C','A')
                                            AND ESONP.NMEVENTO IN ('S-2299','S-2399')
                                            AND ESONP.NRVINCULOM = VM.NRVINCULOM
                                            AND (ESONP.NRTABELA = :P_NRTABELA OR (:P_NRTABELA = 1 AND ESONP.NRTABELA IS NULL) OR :P_NRTABELA IS NULL)
                  LEFT JOIN ESO_NAOPERIODFGTS NPFGTS ON NPFGTS.NRORG = ESONP.NRORG
                                                  AND NPFGTS.NRENVIO = ESONP.NRENVIO                                   
                  LEFT JOIN ESO_PERIODICO ESOPE ON ESOPE.NRORG = VM.NRORG
                                               AND TRUNC(ESOPE.DTMESCOMPETENC, 'MM') = :P_COMPETENCIA
                                               AND ESOPE.NRTPAMBIENTE = EP.NRTPAMBIENTE
                                               AND ESOPE.IDATIVO = 'S' AND ESOPE.NRENVIOEXCL IS NULL
                                               AND ESOPE.IDSTATUS IN('C','A')
                                               AND ESOPE.NMEVENTO IN ('S-1200')
                                               AND ESOPE.NRCHAVE = VM.NRPESSOA
                                               AND (ESOPE.NRTABELA = :P_NRTABELA OR (:P_NRTABELA = 1 AND ESOPE.NRTABELA IS NULL) OR :P_NRTABELA IS NULL)
                  LEFT JOIN ESO_PERIODFGTS PEFGTS ON PEFGTS.NRORG = ESOPE.NRORG
                                               AND PEFGTS.NRENVIO = ESOPE.NRENVIO
                  LEFT JOIN ESTRUTURAH LOT ON LOT.NRORG        = NVL(ESOPE.NRORG,ESONP.NRORG)
                                          AND LOT.NRESTRUTURAM = NVL(ESOPE.NRESTRUTLOTACAO,ESONP.NRESTRUTLOTACAO)
                                          AND LOT.DTMESCOMPETENC = ( SELECT MAX(DTMESCOMPETENC) FROM ESTRUTURAH
                                                                    WHERE DTMESCOMPETENC <= :P_COMPETENCIA
                                                                      AND NRORG        = LOT.NRORG
                                                                      AND NRESTRUTURAM = LOT.NRESTRUTURAM )
                  /*CATEGORIA*/
                  JOIN GPE_VINCULOEMPREG VINCEMPREG ON VINCEMPREG.NRVINCULOEMPREG = VM.NRVINCULOEMPREG
                  JOIN GPE_CATEGTRABESOC CATEGTRAB ON CATEGTRAB.NRCATEGESOC = VINCEMPREG.NRCATEGESOC
                  WHERE VM.NRORG = :P_NRORG
                    AND ( INSTR(','||:P_NRVINCULO||',', ','||VM.NRVINCULOM||',' ) > 0 OR :P_NRVINCULO IS NULL ) --[15-02-12][multisearch_vinculo_156497]
                    AND ( VM.DTADMISSAOVINC <= LAST_DAY(:P_COMPETENCIA) )
                    AND ( VM.DTRESCISAOVINC >= :P_COMPETENCIA OR VM.DTRESCISAOVINC IS NULL )
                    AND ( INSTR(','||:P_NRESTRUTLEGAL||',', ','||VH.NRESTRUTLEGAL||',' ) > 0 OR :P_NRESTRUTLEGAL IS NULL )
                  GROUP BY 
                    VM.NRVINCULOM,
                    PE.NRCPFPESSOA,
                    VM.NMVINCULOM,
                    VH.NRESTRUTLEGAL,
                    EHL.NMRAZSOCESTRUT,
                    EHL.CDCNPJESTRUT,
                    NVL(NPFGTS.VRBASEFGTS13,PEFGTS.VRBASEFGTS13),
                    NVL(ESOPE.DTMESCOMPETENC, ESONP.DTMESCOMPETENC),
                    SUBSTR(EHL.CDCNPJESTRUT, 0,8),
                    NVL(ESOPE.NRESTRUTLOTACAO,ESONP.NRESTRUTLOTACAO),
                    NVL(LOT.CDFPAS,EHL.CDFPAS),
                    NVL(LOT.CDTERCEIRO,EHL.CDTERCEIRO),
                    NVL(NPFGTS.VRBASEFGTSMENSAL,PEFGTS.VRBASEFGTSMENSAL),
                    NVL(NPFGTS.VRBASEFGTS13,PEFGTS.VRBASEFGTS13),
                    VM.DTRESCISAOVINC,
                    CATEGTRAB.DSCATEGESOC,
                    CATEGTRAB.CDESOCIAL
                    ) PRI
      GROUP BY NOME, NRCPFPESSOA, NRLEGAL, RAZAOLEGAL, CNPJLEGAL, PRI.CDFPAS, PRI.CDTERCEIRO, CNPJRAIZ, PRI.NRESTRUTLOTACAO, PRI.VRBASEFGTSMENSAL, PRI.VRBASEFGTS13, PRI.DTRESCISAOVINC, PRI.DSCATEGESOC, PRI.CDESOCIAL)
      ORDER BY CNPJRAIZ, CNPJLEGAL, NOME, DSCATEGESOC";
}
